@@ -20,10 +20,11 @@ from __future__ import print_function
 import numpy as np
 
 from rlgraph.agents import Agent
-from rlgraph.components import Memory, DictMerger, ContainerSplitter, RingBuffer, ValueFunction, Optimizer
+from rlgraph.components import Memory, ContainerMerger, ContainerSplitter, RingBuffer, ValueFunction, Optimizer
 from rlgraph.components.helpers import GeneralizedAdvantageEstimation
 from rlgraph.components.loss_functions.actor_critic_loss_function import ActorCriticLossFunction
 from rlgraph.spaces import FloatBox, BoolBox
+from rlgraph.utils import util
 from rlgraph.utils.decorators import rlgraph_api
 from rlgraph.utils.util import strip_list
 
@@ -34,10 +35,12 @@ class ActorCriticAgent(Agent):
     and entropy regularization. Suitable for execution with A2C, A3C.
     """
 
-    def __init__(self, gae_lambda=1.0, sample_episodes=False, weight_entropy=None, memory_spec=None, **kwargs):
+    def __init__(self, gae_lambda=1.0, clip_rewards=0.0, sample_episodes=False,
+                 weight_entropy=None, memory_spec=None, **kwargs):
         """
         Args:
             gae_lambda (float): Lambda for generalized advantage estimation.
+            clip_rewards (float): Reward clip value. If not 0, rewards will be clipped into this range.
             sample_episodes (bool): If true, the update method interprets the batch_size as the number of
                 episodes to fetch from the memory. If false, batch_size will refer to the number of time-steps. This
                 is especially relevant for environments where episode lengths may vastly differ throughout training. For
@@ -46,8 +49,14 @@ class ActorCriticAgent(Agent):
             memory_spec (Optional[dict,Memory]): The spec for the Memory to use. Should typically be
             a ring-buffer.
         """
+        # Set policy to stochastic.
+        if "policy_spec" in kwargs:
+            policy_spec = kwargs.pop("policy_spec")
+            policy_spec["deterministic"] = False
+        else:
+            policy_spec = dict(deterministic=False)
         super(ActorCriticAgent, self).__init__(
-            policy_spec=dict(deterministic=False),  # Set policy to stochastic.
+            policy_spec=policy_spec,
             name=kwargs.pop("name", "actor-critic-agent"), **kwargs
         )
         self.sample_episodes = sample_episodes
@@ -68,14 +77,15 @@ class ActorCriticAgent(Agent):
         ))
 
         # The merger to merge inputs into one record Dict going into the memory.
-        self.merger = DictMerger("states", "actions", "rewards", "terminals")
+        self.merger = ContainerMerger("states", "actions", "rewards", "terminals")
         self.memory = Memory.from_spec(memory_spec)
         assert isinstance(self.memory, RingBuffer), \
             "ERROR: Actor-critic memory must be ring-buffer for episode-handling."
         # The splitter for splitting up the records coming from the memory.
         self.splitter = ContainerSplitter("states", "actions", "rewards", "terminals")
 
-        self.gae_function = GeneralizedAdvantageEstimation(gae_lambda=gae_lambda, discount=self.discount)
+        self.gae_function = GeneralizedAdvantageEstimation(gae_lambda=gae_lambda, discount=self.discount,
+                                                           clip_rewards=clip_rewards)
         self.loss_function = ActorCriticLossFunction(weight_entropy=weight_entropy)
 
         # Add all our sub-components to the core.
@@ -159,12 +169,12 @@ class ActorCriticAgent(Agent):
             log_probs = agent.policy.get_action_log_probs(preprocessed_states, actions)["action_log_probs"]
             entropy = agent.policy.get_entropy(preprocessed_states)["entropy"]
             loss, loss_per_item, vf_loss, vf_loss_per_item = agent.loss_function.loss(
-                log_probs, entropy, baseline_values, actions, rewards, terminals
+                log_probs, baseline_values, rewards, entropy
             )
 
             # Args are passed in again because some device strategies may want to split them to different devices.
-            policy_vars = agent.policy._variables()
-            vf_vars = agent.value_function._variables()
+            policy_vars = agent.policy.variables()
+            vf_vars = agent.value_function.variables()
 
             step_op, loss, loss_per_item = agent.optimizer.step(policy_vars, loss, loss_per_item)
             vf_step_op, vf_loss, vf_loss_per_item = agent.value_function_optimizer.step(
@@ -250,6 +260,8 @@ class ActorCriticAgent(Agent):
             if sequence_indices is None:
                 sequence_indices = batch["terminals"]
 
+            pps_dtype = self.preprocessed_state_space.dtype
+            batch["states"] = np.asarray(batch["states"], dtype=util.convert_dtype(dtype=pps_dtype, to='np'))
             batch_input = [batch["states"], batch["actions"], batch["rewards"], batch["terminals"], sequence_indices]
 
             # Execute post-processing or already post-processed by workers?
@@ -272,7 +284,7 @@ class ActorCriticAgent(Agent):
         Resets our preprocessor, but only if it contains stateful PreprocessLayer Components (meaning
         the PreprocessorStack has at least one variable defined).
         """
-        if self.preprocessing_required and len(self.preprocessor.variables) > 0:
+        if self.preprocessing_required and len(self.preprocessor.variable_registry) > 0:
             self.graph_executor.execute("reset_preprocessor")
 
     def __repr__(self):
