@@ -24,7 +24,7 @@ import numpy as np
 
 from rlgraph import get_backend
 from rlgraph.components import Component, Exploration, PreprocessorStack, Synchronizable, Policy, Optimizer, \
-    ValueFunction, DictMerger, ContainerSplitter
+    ValueFunction, ContainerMerger, ContainerSplitter
 from rlgraph.graphs.graph_builder import GraphBuilder
 from rlgraph.graphs.graph_executor import GraphExecutor
 from rlgraph.spaces import Space, ContainerSpace
@@ -64,7 +64,7 @@ class Agent(Specifiable):
                 Space object for the Space(s) of the internal (RNN) states.
 
             policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
-            value_function_spec (list): Neural network specification for baseline.
+            value_function_spec (list, dict): Neural network specification for baseline.
 
             exploration_spec (Optional[dict]): The spec-dict to create the Exploration Component.
             execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
@@ -85,17 +85,18 @@ class Agent(Specifiable):
             name (str): Some name for this Agent object.
         """
         super(Agent, self).__init__()
-
         self.name = name
         self.auto_build = auto_build
         self.graph_built = False
         self.logger = logging.getLogger(__name__)
 
         self.state_space = Space.from_spec(state_space).with_batch_rank(False)
-        self.flat_state_space = self.state_space.flatten() if isinstance(self.state_space, ContainerSpace) else None
+        self.flat_state_space = self.state_space.flatten(scope_separator_at_start=False)\
+            if isinstance(self.state_space, ContainerSpace) else None
         self.logger.info("Parsed state space definition: {}".format(self.state_space))
         self.action_space = Space.from_spec(action_space).with_batch_rank(False)
-        self.flat_action_space = self.action_space.flatten() if isinstance(self.action_space, ContainerSpace) else None
+        self.flat_action_space = self.action_space.flatten(scope_separator_at_start=False)\
+            if isinstance(self.action_space, ContainerSpace) else None
         self.logger.info("Parsed action space definition: {}".format(self.action_space))
 
         self.discount = discount
@@ -135,13 +136,17 @@ class Agent(Specifiable):
 
         # Create non-shared baseline network.
         self.value_function = None
+        # TODO move this to specific agents.
         if value_function_spec is not None:
-            self.value_function = ValueFunction(network_spec=value_function_spec)
+            if isinstance(value_function_spec, list):
+                # Use default type if given is list.
+                value_function_spec = dict(type="value_function", network_spec=value_function_spec)
+            self.value_function = ValueFunction.from_spec(value_function_spec)
             self.value_function.add_components(Synchronizable(), expose_apis="sync")
-            self.vars_merger = DictMerger("policy", "vf", scope="variable-dict-merger")
+            self.vars_merger = ContainerMerger("policy", "vf", scope="variable-dict-merger")
             self.vars_splitter = ContainerSplitter("policy", "vf", scope="variable-container-splitter")
         else:
-            self.vars_merger = DictMerger("policy", scope="variable-dict-merger")
+            self.vars_merger = ContainerMerger("policy", scope="variable-dict-merger")
             self.vars_splitter = ContainerSplitter("policy", scope="variable-container-splitter")
 
         self.internal_states_space = Space.from_spec(internal_states_space)
@@ -238,11 +243,11 @@ class Agent(Specifiable):
         @rlgraph_api(component=self.root_component)
         def get_weights(root):
             policy = root.get_sub_component_by_name(agent.policy.scope)
-            policy_weights = policy._variables()
+            policy_weights = policy.variables()
             value_function_weights = None
             if agent.value_function is not None:
                 value_func = root.get_sub_component_by_name(agent.value_function.scope)
-                value_function_weights = value_func._variables()
+                value_function_weights = value_func.variables()
             return dict(policy_weights=policy_weights, value_function_weights=value_function_weights)
 
         @rlgraph_api(component=self.root_component, must_be_complete=False)
@@ -270,6 +275,31 @@ class Agent(Specifiable):
             preprocessor_stack = root.get_sub_component_by_name(agent.preprocessor.scope)
             return preprocessor_stack.preprocess(states)
 
+        @graph_fn(component=self.root_component)
+        def _graph_fn_training_step(root, other_step_op=None):
+            """
+            Increases the global training timestep by 1. Should be called by all training API-methods to
+            timestamp each training/update step.
+
+            Args:
+                other_step_op (Optional[DataOp]): Another DataOp (e.g. a step_op) which should be
+                    executed before the increase takes place.
+
+            Returns:
+                DataOp: no_op.
+            """
+            if get_backend() == "tf":
+                add_op = tf.assign_add(self.graph_executor.global_training_timestep, 1)
+                op_list = [add_op] + [other_step_op] if other_step_op is not None else []
+                with tf.control_dependencies(op_list):
+                    if other_step_op is None or hasattr(other_step_op, "type") and other_step_op.type == "NoOp":
+                        return tf.no_op()
+                    else:
+                        return tf.identity(other_step_op)
+            elif get_backend == "pytorch":
+                self.graph_executor.global_training_timestep += 1
+                return None
+
     def _build_graph(self, root_components, input_spaces, **kwargs):
         """
         Builds the internal graph from the RLGraph meta-graph via the graph executor..
@@ -414,8 +444,14 @@ class Agent(Specifiable):
 
                 # TODO: Apply n-step post-processing if necessary.
                 if self.flat_action_space is not None:
-                    actions_ = {key: np.asarray(self.actions_buffer[env_id][i]) for i, key in
-                                enumerate(self.flat_action_space.keys())}
+                    actions_ = {}
+                    for i, key in enumerate(self.flat_action_space.keys()):
+                        actions_[key] = np.asarray(self.actions_buffer[env_id][i])
+                        # Squeeze, but do not squeeze (1,) to ().
+                        if len(actions_[key]) > 1:
+                            actions_[key] = np.squeeze(actions_[key])
+                        else:
+                            actions_[key] = np.reshape(actions_[key], (1,))
                 else:
                     actions_ = np.asarray(self.actions_buffer[env_id])
                 self._observe_graph(
@@ -465,7 +501,7 @@ class Agent(Specifiable):
                 agent should be configured to sample internally.
 
         Returns:
-            float: The loss value calculated in this update.
+            Union(list, tuple, float): The loss value calculated in this update.
         """
         raise NotImplementedError
 
