@@ -20,7 +20,7 @@ from __future__ import print_function
 import numpy as np
 
 from rlgraph.agents import Agent
-from rlgraph.components import Memory, PrioritizedReplay, DictMerger, ContainerSplitter, DQFDLossFunction
+from rlgraph.components import Memory, PrioritizedReplay, ContainerMerger, ContainerSplitter, DQFDLossFunction
 from rlgraph.spaces import FloatBox, BoolBox
 from rlgraph.utils import RLGraphError
 from rlgraph.utils.decorators import rlgraph_api
@@ -35,7 +35,7 @@ class DQFDAgent(Agent):
     https://arxiv.org/abs/1704.03732
     """
     def __init__(self, expert_margin=0.5, supervised_weight=1.0, double_q=True, dueling_q=True,
-                 huber_loss=False, n_step=1, shared_container_action_target=True,
+                 huber_loss=False, n_step=1, shared_container_action_target=False,
                  memory_spec=None, demo_memory_spec=None,
                  demo_sample_ratio=0.2, store_last_memory_batch=False, store_last_q_table=False, **kwargs):
         # TODO Most of this is DQN duplicate but the way the loss function is instantiated, inheriting
@@ -80,7 +80,13 @@ class DQFDAgent(Agent):
         self.double_q = double_q
         self.dueling_q = dueling_q
         self.huber_loss = huber_loss
-        self.demo_batch_size = int(demo_sample_ratio * self.update_spec['batch_size'] / (1.0 - demo_sample_ratio))
+        self.expert_margin = expert_margin
+
+        self.batch_size = self.update_spec["batch_size"]
+        self.default_margins = np.asarray([self.expert_margin] * self.batch_size)
+
+        self.demo_batch_size = int(demo_sample_ratio * self.update_spec["batch_size"] / (1.0 - demo_sample_ratio))
+        self.demo_margins =  np.asarray([self.expert_margin] * self.demo_batch_size)
         self.shared_container_action_target = shared_container_action_target
 
         # Debugging tools.
@@ -97,7 +103,7 @@ class DQFDAgent(Agent):
 
         self.input_spaces.update(dict(
             actions=self.action_space.with_batch_rank(),
-            weights="variables:{}".format(self.policy.scope),
+            policy_weights="variables:{}".format(self.policy.scope),
             time_step=int,
             use_exploration=bool,
             demo_batch_size=int,
@@ -105,13 +111,14 @@ class DQFDAgent(Agent):
             preprocessed_states=preprocessed_state_space,
             rewards=reward_space,
             terminals=terminal_space,
+            expert_margins=FloatBox(add_batch_rank=True),
             next_states=preprocessed_state_space,
             preprocessed_next_states=preprocessed_state_space,
             importance_weights=weight_space
         ))
 
         # The merger to merge inputs into one record Dict going into the memory.
-        self.merger = DictMerger("states", "actions", "rewards", "next_states", "terminals")
+        self.merger = ContainerMerger("states", "actions", "rewards", "next_states", "terminals")
 
         # The replay memory.
         self.memory = Memory.from_spec(memory_spec)
@@ -129,7 +136,7 @@ class DQFDAgent(Agent):
 
         use_importance_weights = isinstance(self.memory, PrioritizedReplay)
         self.loss_function = DQFDLossFunction(
-            expert_margin=expert_margin, supervised_weight=supervised_weight,
+            supervised_weight=supervised_weight,
             discount=self.discount, double_q=self.double_q, huber_loss=self.huber_loss,
             shared_container_action_target=shared_container_action_target,
             importance_weights=use_importance_weights, n_step=n_step
@@ -196,19 +203,20 @@ class DQFDAgent(Agent):
                 return multi_gpu_syncer.sync_target_qnets()
             # We could be the main root or a multi-GPU tower.
             else:
-                policy_vars = root.get_sub_component_by_name(agent.policy.scope)._variables()
+                policy_vars = root.get_sub_component_by_name(agent.policy.scope).variables()
                 return root.get_sub_component_by_name(agent.target_policy.scope).sync(policy_vars)
 
         # Learn from online memory AND demo memory.
         @rlgraph_api(component=self.root_component)
-        def update_from_memory(root, apply_demo_loss):
+        def update_from_memory(root, apply_demo_loss, expert_margins):
             # Sample from online memory.
-            records, sample_indices, importance_weights = agent.memory.get_records(self.demo_batch_size)
+            records, sample_indices, importance_weights = agent.memory.get_records(self.batch_size)
             preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.split(records)
 
             # Do not apply demo loss to online experience.
             online_step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
-                preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss
+                preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss,
+                expert_margins
             )
 
             if isinstance(agent.memory, PrioritizedReplay):
@@ -220,12 +228,13 @@ class DQFDAgent(Agent):
 
         # Learn from demo-data.
         @rlgraph_api(component=self.root_component)
-        def update_from_demos(root, demo_batch_size, apply_demo_loss):
+        def update_from_demos(root, demo_batch_size, apply_demo_loss, expert_margins):
             # Sample from demo memory.
             records, sample_indices, importance_weights = agent.demo_memory.get_records(demo_batch_size)
             preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.split(records)
             step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
-                preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss
+                preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss,
+                expert_margins
             )
             return step_op, loss, loss_per_item, records, q_values_s
 
@@ -233,19 +242,21 @@ class DQFDAgent(Agent):
         @rlgraph_api(component=self.root_component)
         def update_from_external_batch(
                 root, preprocessed_states, actions, rewards, terminals, preprocessed_next_states, importance_weights,
-                apply_demo_loss
+                apply_demo_loss, expert_margins
         ):
             # If we are a multi-GPU root:
             # Simply feeds everything into the multi-GPU sync optimizer's method and return.
             if "multi-gpu-synchronizer" in root.sub_components:
-                main_policy_vars = agent.policy._variables()
+                main_policy_vars = agent.policy.variables()
                 # TODO: This may be called differently in other agents (replace by root-policy).
                 grads_and_vars, loss, loss_per_item, q_values_s = \
                     root.sub_components["multi-gpu-synchronizer"].calculate_update_from_external_batch(
                         dict(policy=main_policy_vars), preprocessed_states, actions, rewards, terminals,
-                        preprocessed_next_states, importance_weights, apply_demo_loss
+                        preprocessed_next_states, importance_weights, apply_demo_loss, expert_margins
                     )
                 step_op = agent.optimizer.apply_gradients(grads_and_vars)
+                # Increase the global training step counter.
+                step_op = root._graph_fn_training_step(step_op)
                 step_and_sync_op = root.sub_components["multi-gpu-synchronizer"].sync_variables_to_towers(
                     step_op, main_policy_vars
                 )
@@ -258,44 +269,48 @@ class DQFDAgent(Agent):
             optimizer = root.get_sub_component_by_name(agent.optimizer.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_probabilities_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
+            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             loss, loss_per_item = loss_function.loss(
-                q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp, importance_weights, apply_demo_loss
+                q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins, q_values_sp,
+                importance_weights, apply_demo_loss
             )
 
             # Args are passed in again because some device strategies may want to split them to different devices.
-            policy_vars = policy._variables()
+            policy_vars = policy.variables()
             if hasattr(root, "is_multi_gpu_tower") and root.is_multi_gpu_tower is True:
                 grads_and_vars = optimizer.calculate_gradients(policy_vars, loss)
                 return grads_and_vars, loss, loss_per_item, q_values_s
             else:
                 step_op, loss, loss_per_item = optimizer.step(policy_vars, loss, loss_per_item)
+                # Increase the global training step counter.
+                step_op = root._graph_fn_training_step(step_op)
                 return step_op, loss, loss_per_item, q_values_s
 
         @rlgraph_api(component=self.root_component)
         def get_td_loss(root, preprocessed_states, actions, rewards,
-                        terminals, preprocessed_next_states, importance_weights, apply_demo_loss):
+                        terminals, preprocessed_next_states, importance_weights, apply_demo_loss, expert_margins):
 
             policy = root.get_sub_component_by_name(agent.policy.scope)
             target_policy = root.get_sub_component_by_name(agent.target_policy.scope)
             loss_function = root.get_sub_component_by_name(agent.loss_function.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_probabilities_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
+            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             loss, loss_per_item = loss_function.loss(
-                q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp, importance_weights, apply_demo_loss
+                q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins,
+                q_values_sp, importance_weights, apply_demo_loss
             )
             return loss, loss_per_item
 
@@ -343,7 +358,24 @@ class DQFDAgent(Agent):
     def _observe_graph(self, preprocessed_states, actions, internals, rewards, next_states, terminals):
         self.graph_executor.execute(("insert_records", [preprocessed_states, actions, rewards, next_states, terminals]))
 
-    def update(self, batch=None):
+    def update(self, batch=None, update_from_demos=False, expert_margins=None, apply_demo_loss_to_batch=False):
+        """
+        Updates from external batch or replay memory.
+        Args:
+            batch (Optional[dict]): Optional dict to use for updating. If false, samples from replay memory
+            update_from_demos (bool): If true, also updates from demo memory by sampling demonstrations from
+                demo memory. Default false (only updating from main replay memory).
+            expert_margins (SingleDataOp): The expert margin enforces a distance in Q-values between expert action and
+                all other actions.
+           apply_demo_loss_to_batch (bool): If true and an external batch is given, demo loss is applied to this
+                batch. Can be combined with external per-sample expert margins. The purpose of this is to update
+                from external demonstration data where each sample has a different margin. If false and an
+                external batch is given, the agent updates this with the normal Double/Dueling-q loss. Default
+                false. Only valid if batch is not None.
+        Returns:
+            tuple: Loss and loss per item.
+        """
+
         # Should we sync the target net?
         self.steps_since_target_net_sync += self.update_spec["update_interval"]
         if self.steps_since_target_net_sync >= self.update_spec["sync_interval"]:
@@ -356,6 +388,7 @@ class DQFDAgent(Agent):
         return_ops = [0, 1, 2]
         q_table = None
 
+        # Update from replay memory.  Potentially also update from demo memory with default margins.
         if batch is None:
             # Add some additional return-ops to pull (left out normally for performance reasons).
             if self.store_last_q_table is True:
@@ -364,9 +397,16 @@ class DQFDAgent(Agent):
                 return_ops += [3]  # 3=batch
 
             # Combine: Update from memory (apply_demo_loss=False), update_from_demo (apply=True).
-            ret = self.graph_executor.execute(("update_from_memory", False, return_ops),
-                                              ("update_from_demos", [self.demo_batch_size, True], return_ops),
-                                              sync_call)
+            # Otherwise only update from online memory.
+            if update_from_demos:
+                # Use default margins whe sampling from memory.
+                ret = self.graph_executor.execute(("update_from_memory", [False, self.default_margins], return_ops),
+                                                  ("update_from_demos", [self.demo_batch_size, True,
+                                                                         self.demo_margins], return_ops),
+                                                  sync_call)
+            else:
+                ret = self.graph_executor.execute(("update_from_memory",  [False, self.default_margins], return_ops),
+                                                  sync_call)
 
             # Remove unnecessary return dicts (e.g. sync-op).
             if isinstance(ret, dict):
@@ -379,16 +419,30 @@ class DQFDAgent(Agent):
                     q_values=ret[4]
                 )
         else:
+            # Update from external batch, optionally applying demo loss. Also optionally
+            # sample demos from separate demo memory.
+
             # Add some additional return-ops to pull (left out normally for performance reasons).
             if self.store_last_q_table is True:
                 return_ops += [3]  # 3=q-values
 
-            # Add false: no demo loss.
+            if expert_margins is None:
+                # Default margins with correct len.
+                expert_margins = np.asarray([self.expert_margin] * len(batch["terminals"]))
+
+            # Apply demo loss to external flag depending on batch.
+            # Expert margins only have effect if True.
             batch_input = [batch["states"], batch["actions"], batch["rewards"], batch["terminals"],
-                           batch["next_states"], batch["importance_weights"], False]
-            ret = self.graph_executor.execute(("update_from_external_batch", batch_input, return_ops),
-                                              ("update_from_demos", [self.demo_batch_size, True], return_ops),
-                                              sync_call)
+                           batch["next_states"], batch["importance_weights"], apply_demo_loss_to_batch, expert_margins]
+
+            if update_from_demos:
+                ret = self.graph_executor.execute(("update_from_external_batch", batch_input, return_ops),
+                                                  ("update_from_demos", [self.demo_batch_size, True,
+                                                                         self.demo_margins], return_ops),
+                                                  sync_call)
+            else:
+                # Only update from external batch (which may use demo loss depending on flag.
+                ret = self.graph_executor.execute(("update_from_external_batch", batch_input, return_ops), sync_call)
 
             # Remove unnecessary return dicts (e.g. sync-op).
             if isinstance(ret, dict):
@@ -416,21 +470,25 @@ class DQFDAgent(Agent):
         Resets our preprocessor, but only if it contains stateful PreprocessLayer Components (meaning
         the PreprocessorStack has at least one variable defined).
         """
-        if self.preprocessing_required and len(self.preprocessor.variables) > 0:
+        if self.preprocessing_required and len(self.preprocessor.variable_registry) > 0:
             self.graph_executor.execute("reset_preprocessor")
 
     def update_from_demos(self, num_updates=1, batch_size=None):
         """
         Executes a number of updates by sampling from the expert memory.
+
         Args:
             num_updates (int): The number of samples to execute.
             batch_size (Optional[int]): Sampling batch size to use. If None, uses the demo
-                batch size computed via the sampling ratio /
+                batch size computed via the sampling ratio.
         """
         if batch_size is None:
             batch_size = self.demo_batch_size
+            margins = self.demo_margins
+        else:
+            margins = np.asarray([self.expert_margin] * batch_size)
         for _ in range(num_updates):
-            self.graph_executor.execute(("update_from_demos", [batch_size, True]))
+            self.graph_executor.execute(("update_from_demos", [batch_size, True, margins]))
 
     def observe_demos(self, preprocessed_states, actions, rewards, next_states, terminals):
         """
