@@ -20,7 +20,7 @@ from __future__ import print_function
 import numpy as np
 
 from rlgraph.agents import Agent
-from rlgraph.components import Memory, PrioritizedReplay, DQNLossFunction, DictMerger, ContainerSplitter
+from rlgraph.components import Memory, PrioritizedReplay, DQNLossFunction, ContainerMerger, ContainerSplitter
 from rlgraph.spaces import FloatBox, BoolBox
 from rlgraph.utils import RLGraphError
 from rlgraph.utils.decorators import rlgraph_api
@@ -106,12 +106,13 @@ class DQNAgent(Agent):
             next_states=preprocessed_state_space,
             preprocessed_next_states=preprocessed_state_space,
             importance_weights=weight_space,
+            apply_postprocessing=bool
         ))
         if self.value_function is not None:
             self.input_spaces["value_function_weights"] = "variables:{}".format(self.value_function.scope),
 
         # The merger to merge inputs into one record Dict going into the memory.
-        self.merger = DictMerger("states", "actions", "rewards", "next_states", "terminals")
+        self.merger = ContainerMerger("states", "actions", "rewards", "next_states", "terminals")
         # The replay memory.
         self.memory = Memory.from_spec(memory_spec)
         # The splitter for splitting up the records coming from the memory.
@@ -191,21 +192,22 @@ class DQNAgent(Agent):
                 return multi_gpu_syncer.sync_target_qnets()
             # We could be the main root or a multi-GPU tower.
             else:
-                policy_vars = root.get_sub_component_by_name(agent.policy.scope)._variables()
+                policy_vars = root.get_sub_component_by_name(agent.policy.scope).variables()
                 return root.get_sub_component_by_name(agent.target_policy.scope).sync(policy_vars)
 
         # Learn from memory.
         @rlgraph_api(component=self.root_component)
-        def update_from_memory(root):
+        def update_from_memory(root, apply_postprocessing):
             # Non prioritized memory will just return weight 1.0 for all samples.
             records, sample_indices, importance_weights = agent.memory.get_records(agent.update_spec["batch_size"])
             preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.split(records)
 
             step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
-                preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights
+                preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights,
+                apply_postprocessing
             )
 
-            # TODO this is really annoying.. will be solved once we have dict returns.
+            # TODO this is really annoying. Will be solved once we have dict returns.
             if isinstance(agent.memory, PrioritizedReplay):
                 update_pr_step_op = agent.memory.update_records(sample_indices, loss_per_item)
                 return step_op, loss, loss_per_item, records, q_values_s, update_pr_step_op
@@ -216,19 +218,21 @@ class DQNAgent(Agent):
         @rlgraph_api(component=self.root_component)
         def update_from_external_batch(
                 root, preprocessed_states, actions, rewards, terminals, preprocessed_next_states,
-                importance_weights
+                importance_weights, apply_postprocessing
         ):
             # If we are a multi-GPU root:
             # Simply feeds everything into the multi-GPU sync optimizer's method and return.
             if "multi-gpu-synchronizer" in root.sub_components:
-                main_policy_vars = agent.policy._variables()
+                main_policy_vars = agent.policy.variables()
                 all_vars = agent.vars_merger.merge(main_policy_vars)
                 out = root.sub_components["multi-gpu-synchronizer"].calculate_update_from_external_batch(
                     all_vars, preprocessed_states, actions, rewards, terminals,
-                    preprocessed_next_states, importance_weights
+                    preprocessed_next_states, importance_weights, apply_postprocessing=apply_postprocessing
                 )
                 avg_grads_and_vars = agent.vars_splitter.split(out["avg_grads_and_vars_by_component"])
                 step_op = agent.optimizer.apply_gradients(avg_grads_and_vars)
+                # Increase the global training step counter.
+                step_op = root._graph_fn_training_step(step_op)
                 step_and_sync_op = root.sub_components["multi-gpu-synchronizer"].sync_variables_to_towers(
                     step_op, all_vars
                 )
@@ -243,19 +247,19 @@ class DQNAgent(Agent):
             vars_merger = root.get_sub_component_by_name(agent.vars_merger.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_probabilities_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
+            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp, importance_weights
             )
 
             # Args are passed in again because some device strategies may want to split them to different devices.
-            policy_vars = policy._variables()
+            policy_vars = policy.variables()
 
             # TODO: for a fully automated multi-GPU strategy, we would have to make sure that:
             # TODO: - every agent (root_component) has an update_from_external_batch method
@@ -266,6 +270,8 @@ class DQNAgent(Agent):
                 return grads_and_vars_by_component, loss, loss_per_item, q_values_s
             else:
                 step_op, loss, loss_per_item = optimizer.step(policy_vars, loss, loss_per_item)
+                # Increase the global training step counter.
+                step_op = root._graph_fn_training_step(step_op)
                 return step_op, loss, loss_per_item, q_values_s
 
         @rlgraph_api(component=self.root_component)
@@ -277,12 +283,12 @@ class DQNAgent(Agent):
             loss_function = root.get_sub_component_by_name(agent.loss_function.scope)
 
             # Get the different Q-values.
-            q_values_s = policy.get_logits_probabilities_log_probs(preprocessed_states)["logits"]
-            qt_values_sp = target_policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+            q_values_s = policy.get_logits_parameters_log_probs(preprocessed_states)["logits"]
+            qt_values_sp = target_policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
+                q_values_sp = policy.get_logits_parameters_log_probs(preprocessed_next_states)["logits"]
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp, importance_weights
@@ -352,7 +358,7 @@ class DQNAgent(Agent):
                 return_ops += [3, 4]  # 3=batch, 4=q-values
             elif self.store_last_memory_batch is True:
                 return_ops += [3]  # 3=batch
-            ret = self.graph_executor.execute(("update_from_memory", None, return_ops))
+            ret = self.graph_executor.execute(("update_from_memory", [True], return_ops))
 
             # Store the last Q-table?
             if self.store_last_q_table is True:
@@ -365,8 +371,9 @@ class DQNAgent(Agent):
             if self.store_last_q_table is True:
                 return_ops += [3]  # 3=q-values
 
+            # TODO apply postprocessing always true atm.
             batch_input = [batch["states"], batch["actions"], batch["rewards"], batch["terminals"],
-                           batch["next_states"], batch["importance_weights"]]
+                           batch["next_states"], batch["importance_weights"], True]
             ret = self.graph_executor.execute(("update_from_external_batch", batch_input, return_ops))
 
             # Store the last Q-table?
@@ -396,7 +403,7 @@ class DQNAgent(Agent):
         Resets our preprocessor, but only if it contains stateful PreprocessLayer Components (meaning
         the PreprocessorStack has at least one variable defined).
         """
-        if self.preprocessing_required and len(self.preprocessor.variables) > 0:
+        if self.preprocessing_required and len(self.preprocessor.variable_registry) > 0:
             self.graph_executor.execute("reset_preprocessor")
 
     def post_process(self, batch):
