@@ -18,11 +18,12 @@ from __future__ import division
 from __future__ import print_function
 
 import copy
+import tensorflow as tf
 
-from rlgraph.utils.decorators import rlgraph_api
+from rlgraph.utils.decorators import rlgraph_api, graph_fn
 from rlgraph.utils import RLGraphError
 from rlgraph.agents.agent import Agent
-from rlgraph.components.common.dict_merger import DictMerger
+from rlgraph.components.common.container_merger import ContainerMerger
 from rlgraph.components.common.container_splitter import ContainerSplitter
 from rlgraph.components.common.slice import Slice
 from rlgraph.components.common.staging_area import StagingArea
@@ -243,7 +244,7 @@ class IMPALAAgent(Agent):
             # Slice some data from the EnvStepper (e.g only first internal states are needed).
             self.internal_states_slicer = Slice(scope="internal-states-slicer", squeeze=True)
             # Merge back to insert into FIFO.
-            self.fifo_input_merger = DictMerger(*self.fifo_queue_keys)
+            self.fifo_input_merger = ContainerMerger(*self.fifo_queue_keys)
 
             # Dummy Flattener to calculate action-probs space.
             dummy_flattener = ReShape(flatten=True, flatten_categories=self.action_space.num_categories)
@@ -375,7 +376,7 @@ class IMPALAAgent(Agent):
         """
         # Perform n-steps in the env and insert the results into our FIFO-queue.
         @rlgraph_api(component=self.root_component)
-        def perform_n_steps_and_insert_into_fifo(self_):
+        def perform_n_steps_and_insert_into_fifo(root):
             # Take n steps in the environment.
             step_results = env_stepper.step()
 
@@ -419,11 +420,11 @@ class IMPALAAgent(Agent):
             optimizer (Optimizer): The optimizer that we use to calculate an update and apply it.
         """
         @rlgraph_api(component=self.root_component)
-        def get_queue_size(self_):
+        def get_queue_size(root):
             return fifo_queue.get_size()
 
         @rlgraph_api(component=self.root_component)
-        def update_from_memory(self_):
+        def update_from_memory(root):
             # Pull n records from the queue.
             # Note that everything will come out as batch-major and must be transposed before the main-LSTM.
             # This is done by the network itself for all network inputs:
@@ -465,7 +466,7 @@ class IMPALAAgent(Agent):
             preprocessed_states = preprocessor.preprocess(states)
 
             # Only retrieve logits and do faster sparse softmax in loss.
-            out = policy.get_state_values_logits_probabilities_log_probs(preprocessed_states, initial_internal_states)
+            out = policy.get_state_values_logits_parameters_log_probs(preprocessed_states, initial_internal_states)
             state_values_pi = out["state_values"]
             logits = out["logits"]
             #current_internal_states = out["last_internal_states"]
@@ -480,14 +481,17 @@ class IMPALAAgent(Agent):
             loss, loss_per_item = loss_function.loss(
                 logits, action_probs_mu, state_values_pi, actions, rewards, terminals
             )
-            policy_vars = policy._variables()
+            policy_vars = policy.variables()
 
             # Pass vars and loss values into optimizer.
             step_op, loss, loss_per_item = optimizer.step(policy_vars, loss, loss_per_item)
 
+            # Increase the global training step counter.
+            step_op = root._graph_fn_training_step(step_op)
+
             # Return optimizer op and all loss values.
             # TODO: Make it possible to return None from API-method without messing with the meta-graph.
-            return step_op, (stage_op if stage_op else step_op), loss, loss_per_item
+            return step_op, (stage_op if stage_op else step_op), loss, loss_per_item, records
 
     def get_action(self, states, internal_states=None, use_exploration=True, extra_returns=None):
         pass
@@ -595,7 +599,7 @@ class SingleIMPALAAgent(IMPALAAgent):
         )
 
         # Merge back to insert into FIFO.
-        self.fifo_input_merger = DictMerger(*self.fifo_queue_keys)
+        self.fifo_input_merger = ContainerMerger(*self.fifo_queue_keys)
 
         # Dummy Flattener to calculate action-probs space.
         dummy_flattener = ReShape(flatten=True, flatten_categories=self.action_space.num_categories)
@@ -680,7 +684,6 @@ class SingleIMPALAAgent(IMPALAAgent):
                     out_op_columns[0].op_records[0].op
 
     def define_graph_api(self):
-
         agent = self
 
         @rlgraph_api(component=self.root_component)
@@ -740,7 +743,7 @@ class SingleIMPALAAgent(IMPALAAgent):
                 states = agent.preprocessor.preprocess(states)
 
             # Get the pi-action probs AND the values for all our states.
-            out = agent.policy.get_state_values_logits_probabilities_log_probs(states, initial_internal_states)
+            out = agent.policy.get_state_values_logits_parameters_log_probs(states, initial_internal_states)
             state_values_pi = out["state_values"]
             logits_pi = out["logits"]
 
@@ -756,16 +759,26 @@ class SingleIMPALAAgent(IMPALAAgent):
                 logits_pi, action_probs_mu, state_values_pi, actions, rewards, terminals
             )
             if self.dynamic_batching:
-                policy_vars = agent.queue_runner.data_producing_components[0].actor_component.policy._variables()
+                policy_vars = agent.queue_runner.data_producing_components[0].actor_component.policy.variables()
             else:
-                policy_vars = agent.policy._variables()
+                policy_vars = agent.policy.variables()
 
             # Pass vars and loss values into optimizer.
             step_op, loss, loss_per_item = agent.optimizer.step(policy_vars, loss, loss_per_item)
+            # Increase the global training step counter.
+            step_op = root._graph_fn_training_step(step_op)
 
             # Return optimizer op and all loss values.
             # TODO: Make it possible to return None from API-method without messing with the meta-graph.
             return step_op, (stage_op if stage_op else step_op), loss, loss_per_item, records
 
+        # TODO: Move this into generic AgentRootComponent.
+        @graph_fn(component=self.root_component)
+        def _graph_fn_training_step(root, other_step_op=None):
+            add_op = tf.assign_add(self.graph_executor.global_training_timestep, 1)
+            op_list = [add_op] + [other_step_op] if other_step_op is not None else []
+            with tf.control_dependencies(op_list):
+                return tf.no_op() if other_step_op is None else other_step_op
+
     def __repr__(self):
         return "SingleIMPALAAgent()"
