@@ -21,11 +21,13 @@ import numpy as np
 
 from rlgraph import get_backend
 from rlgraph.agents import Agent
-from rlgraph.components import DictMerger, ContainerSplitter, Memory, RingBuffer, PPOLossFunction, ValueFunction, \
-    Optimizer
+from rlgraph.components import ContainerMerger, ContainerSplitter, Memory, RingBuffer, PPOLossFunction
 from rlgraph.components.helpers import GeneralizedAdvantageEstimation
 from rlgraph.spaces import BoolBox, FloatBox
-from rlgraph.utils.util import strip_list, default_dict
+from rlgraph.utils import util
+from rlgraph.utils.execution_util import define_by_run_flatten
+from rlgraph.utils.ops import flatten_op, DataOpDict
+from rlgraph.utils.util import strip_list
 from rlgraph.utils.decorators import rlgraph_api
 
 if get_backend() == "tf":
@@ -43,12 +45,13 @@ class PPOAgent(Agent):
     Paper: https://arxiv.org/abs/1707.06347
     """
 
-    def __init__(self, clip_ratio=0.2, gae_lambda=1.0, standardize_advantages=False,
+    def __init__(self, clip_ratio=0.2, gae_lambda=1.0, clip_rewards=0.0, standardize_advantages=False,
                  sample_episodes=True, weight_entropy=None, memory_spec=None, **kwargs):
         """
         Args:
             clip_ratio (float): Clipping parameter for likelihood ratio.
             gae_lambda (float): Lambda for generalized advantage estimation.
+            clip_rewards (float): Reward clip value. If not 0, rewards will be clipped into this range.
             standardize_advantages (bool): If true, standardize advantage values in update.
 
             sample_episodes (bool): If True, the update method interprets the batch_size as the number of
@@ -61,8 +64,13 @@ class PPOAgent(Agent):
             memory_spec (Optional[dict,Memory]): The spec for the Memory to use. Should typically be
                 a ring-buffer.
         """
+        if "policy_spec" in kwargs:
+            policy_spec = kwargs.pop("policy_spec")
+            policy_spec["deterministic"] = False
+        else:
+            policy_spec = dict(deterministic=False)
         super(PPOAgent, self).__init__(
-            policy_spec=dict(deterministic=False),  # Set policy to stochastic.
+            policy_spec=policy_spec,  # Set policy to stochastic.
             name=kwargs.pop("name", "ppo-agent"), **kwargs
         )
         self.sample_episodes = sample_episodes
@@ -89,7 +97,7 @@ class PPOAgent(Agent):
         ))
 
         # The merger to merge inputs into one record Dict going into the memory.
-        self.merger = DictMerger("states", "actions", "rewards", "terminals")
+        self.merger = ContainerMerger("states", "actions", "rewards", "terminals")
         self.memory = Memory.from_spec(memory_spec)
         assert isinstance(self.memory, RingBuffer), "ERROR: PPO memory must be ring-buffer for episode-handling!"
 
@@ -100,7 +108,8 @@ class PPOAgent(Agent):
 
         # The splitter for splitting up the records coming from the memory.
         self.splitter = ContainerSplitter("states", "actions", "rewards", "terminals")
-        self.gae_function = GeneralizedAdvantageEstimation(gae_lambda=gae_lambda, discount=self.discount)
+        self.gae_function = GeneralizedAdvantageEstimation(gae_lambda=gae_lambda, discount=self.discount,
+                                                           clip_rewards=clip_rewards)
         self.loss_function = PPOLossFunction(clip_ratio=clip_ratio,
                                              standardize_advantages=standardize_advantages,
                                              weight_entropy=weight_entropy)
@@ -201,15 +210,31 @@ class PPOAgent(Agent):
             value_function_optimizer = root.get_sub_component_by_name(agent.value_function_optimizer.scope)
             vars_merger = root.get_sub_component_by_name(agent.vars_merger.scope)
             gae_function = root.get_sub_component_by_name(agent.gae_function.scope)
+            prev_log_probs = policy.get_action_log_probs(preprocessed_states, actions)["action_log_probs"]
 
             if get_backend() == "tf":
+                # Log probs before update.
+                if isinstance(prev_log_probs, DataOpDict):
+                    for name in prev_log_probs.keys():
+                        prev_log_probs[name] = tf.stop_gradient(prev_log_probs[name])
+                else:
+                    prev_log_probs = tf.stop_gradient(prev_log_probs)
                 batch_size = tf.shape(preprocessed_states)[0]
 
                 def opt_body(index_, loss_, loss_per_item_, vf_loss_, vf_loss_per_item_):
                     start = tf.random_uniform(shape=(), minval=0, maxval=batch_size - 1, dtype=tf.int32)
                     indices = tf.range(start=start, limit=start + agent.sample_size) % batch_size
                     sample_states = tf.gather(params=preprocessed_states, indices=indices)
-                    sample_actions = tf.gather(params=actions, indices=indices)
+                    if isinstance(actions, dict):
+                        sample_actions = DataOpDict()
+                        sample_prior_log_probs = DataOpDict()
+                        for name, action in flatten_op(actions, scope_separator_at_start=False).items():
+                            sample_actions[name] = tf.gather(params=action, indices=indices)
+                            sample_prior_log_probs[name] = tf.gather(params=prev_log_probs[name], indices=indices)
+                    else:
+                        sample_actions = tf.gather(params=actions, indices=indices)
+                        sample_prior_log_probs = tf.gather(params=prev_log_probs, indices=indices)
+
                     sample_rewards = tf.gather(params=rewards, indices=indices)
                     sample_terminals = tf.gather(params=terminals, indices=indices)
                     sample_sequence_indices = tf.gather(params=sequence_indices, indices=indices)
@@ -217,14 +242,14 @@ class PPOAgent(Agent):
                     # If we are a multi-GPU root:
                     # Simply feeds everything into the multi-GPU sync optimizer's method and return.
                     if multi_gpu_sync_optimizer is not None:
-                        main_policy_vars = agent.policy._variables()
-                        main_vf_vars = agent.value_function._variables()
+                        main_policy_vars = agent.policy.variables()
+                        main_vf_vars = agent.value_function.variables()
                         all_vars = agent.vars_merger.merge(main_policy_vars, main_vf_vars)
                         # grads_and_vars, loss, loss_per_item, vf_loss, vf_loss_per_item = \
                         out = multi_gpu_sync_optimizer.calculate_update_from_external_batch(
                             all_vars,
                             sample_states, sample_actions, sample_rewards, sample_terminals, sample_sequence_indices,
-                            apply_postprocessing
+                            apply_postprocessing=apply_postprocessing
                         )
                         avg_grads_and_vars_policy, avg_grads_and_vars_vf = agent.vars_splitter.split(
                             out["avg_grads_and_vars_by_component"]
@@ -244,6 +269,9 @@ class PPOAgent(Agent):
                         out["loss_per_item"].set_shape((agent.sample_size,))
 
                         with tf.control_dependencies([step_and_sync_op]):
+                            if index_ == 0:
+                                # Increase the global training step counter.
+                                out["loss"] = root._graph_fn_training_step(out["loss"])
                             return index_ + 1, out["loss"], out["loss_per_item"], loss_vf, loss_per_item_vf
 
                     policy_probs = policy.get_action_log_probs(sample_states, sample_actions)
@@ -254,30 +282,31 @@ class PPOAgent(Agent):
                             baseline_values, sample_rewards, sample_terminals, sample_sequence_indices),
                         false_fn=lambda: sample_rewards
                     )
+                    sample_rewards.set_shape((agent.sample_size, ))
                     entropy = policy.get_entropy(sample_states)["entropy"]
 
                     loss, loss_per_item, vf_loss, vf_loss_per_item = \
                         loss_function.loss(
-                            policy_probs["action_log_probs"], baseline_values, actions, sample_rewards,
-                            sample_terminals, entropy
+                            policy_probs["action_log_probs"], sample_prior_log_probs,
+                            baseline_values, sample_rewards,  entropy
                         )
 
                     if hasattr(root, "is_multi_gpu_tower") and root.is_multi_gpu_tower is True:
-                        policy_grads_and_vars = optimizer.calculate_gradients(policy._variables(), loss)
+                        policy_grads_and_vars = optimizer.calculate_gradients(policy.variables(), loss)
                         vf_grads_and_vars = value_function_optimizer.calculate_gradients(
-                            value_function._variables(), vf_loss
+                            value_function.variables(), vf_loss
                         )
                         grads_and_vars_by_component = vars_merger.merge(policy_grads_and_vars, vf_grads_and_vars)
                         return grads_and_vars_by_component, loss, loss_per_item, vf_loss, vf_loss_per_item
                     else:
                         step_op, loss, loss_per_item = optimizer.step(
-                            policy._variables(), loss, loss_per_item
+                            policy.variables(), loss, loss_per_item
                         )
                         loss.set_shape(())
                         loss_per_item.set_shape((agent.sample_size,))
 
                         vf_step_op, vf_loss, vf_loss_per_item = value_function_optimizer.step(
-                            value_function._variables(), vf_loss, vf_loss_per_item
+                            value_function.variables(), vf_loss, vf_loss_per_item
                         )
                         vf_loss.set_shape(())
                         vf_loss_per_item.set_shape((agent.sample_size,))
@@ -305,9 +334,16 @@ class PPOAgent(Agent):
                         loop_vars=init_loop_vars,
                         parallel_iterations=1
                     )
+                    # Increase the global training step counter.
+                    loss = root._graph_fn_training_step(loss)
                     return loss, loss_per_item, vf_loss, vf_loss_per_item
 
             elif get_backend() == "pytorch":
+                if isinstance(prev_log_probs, dict):
+                    for name in actions.keys():
+                        prev_log_probs[name] = prev_log_probs[name].detach()
+                else:
+                    prev_log_probs = prev_log_probs.detach()
                 batch_size = preprocessed_states.shape[0]
                 sample_size = min(batch_size, agent.sample_size)
 
@@ -315,10 +351,21 @@ class PPOAgent(Agent):
                     start = int(torch.rand(1) * (batch_size - 1))
                     indices = torch.arange(start=start, end=start + sample_size, dtype=torch.long) % batch_size
                     sample_states = torch.index_select(preprocessed_states, 0, indices)
-                    sample_actions = torch.index_select(actions, 0, indices)
+
+                    if isinstance(actions, dict):
+                        sample_actions = DataOpDict()
+                        sample_prior_log_probs = DataOpDict()
+                        for name, action in define_by_run_flatten(actions, scope_separator_at_start=False).items():
+                            sample_actions[name] = torch.index_select(action, 0, indices)
+                            sample_prior_log_probs[name] = torch.index_select(prev_log_probs[name], 0, indices)
+                    else:
+                        sample_actions = torch.index_select(actions, 0, indices)
+                        sample_prior_log_probs = torch.index_select(prev_log_probs, 0, indices)
+
                     sample_rewards = torch.index_select(rewards, 0, indices)
                     sample_terminals = torch.index_select(terminals, 0, indices)
                     sample_sequence_indices = torch.index_select(sequence_indices, 0, indices)
+
                     policy_probs = policy.get_action_log_probs(sample_states, sample_actions)
 
                     baseline_values = value_function.value_output(sample_states)
@@ -328,15 +375,15 @@ class PPOAgent(Agent):
 
                     entropy = policy.get_entropy(sample_states)["entropy"]
                     loss, loss_per_item, vf_loss, vf_loss_per_item = loss_function.loss(
-                        policy_probs["action_log_probs"], baseline_values, actions, sample_rewards,
-                        sample_terminals, entropy
+                        policy_probs["action_log_probs"], sample_prior_log_probs,
+                        baseline_values,  sample_rewards, entropy
                     )
 
                     # Do not need step op.
-                    _, loss, loss_per_item = optimizer.step(policy._variables(), loss, loss_per_item)
+                    _, loss, loss_per_item = optimizer.step(policy.variables(), loss, loss_per_item)
 
                     _, vf_loss, vf_loss_per_item = \
-                        value_function_optimizer.step(value_function._variables(), vf_loss, vf_loss_per_item)
+                        value_function_optimizer.step(value_function.variables(), vf_loss, vf_loss_per_item)
 
                 return loss, loss_per_item, vf_loss, vf_loss_per_item
 
@@ -419,6 +466,8 @@ class PPOAgent(Agent):
             if sequence_indices is None:
                 sequence_indices = batch["terminals"]
 
+            pps_dtype = self.preprocessed_state_space.dtype
+            batch["states"] = np.asarray(batch["states"], dtype=util.convert_dtype(dtype=pps_dtype, to='np'))
             batch_input = [batch["states"], batch["actions"], batch["rewards"], batch["terminals"],
                            sequence_indices, apply_postprocessing]
 
@@ -442,7 +491,7 @@ class PPOAgent(Agent):
         Resets our preprocessor, but only if it contains stateful PreprocessLayer Components (meaning
         the PreprocessorStack has at least one variable defined).
         """
-        if self.preprocessing_required and len(self.preprocessor.variables) > 0:
+        if self.preprocessing_required and len(self.preprocessor.variable_registry) > 0:
             self.graph_executor.execute("reset_preprocessor")
 
     def post_process(self, batch):
