@@ -24,13 +24,14 @@ import copy
 import inspect
 import numpy as np
 import re
+import uuid
 
 from rlgraph import get_backend
 from rlgraph.utils.decorators import rlgraph_api, component_api_registry, component_graph_fn_registry,\
     define_api_method, define_graph_fn
-from rlgraph.utils.rlgraph_errors import RLGraphError
+from rlgraph.utils.rlgraph_errors import RLGraphError, RLGraphObsoletedError
 from rlgraph.utils.specifiable import Specifiable
-from rlgraph.utils.ops import DataOpDict, FLAT_TUPLE_OPEN, FLAT_TUPLE_CLOSE
+from rlgraph.utils.ops import DataOpDict, FLAT_TUPLE_OPEN, FLAT_TUPLE_CLOSE, TraceContext
 from rlgraph.utils import util
 
 if get_backend() == "tf":
@@ -99,7 +100,7 @@ class Component(Specifiable):
         # self.logger = logging.getLogger(__name__)
         self.scope = kwargs.pop("scope", "")
 
-        assert re.match(r'^[\w\-]*$', self.scope), \
+        assert re.match(r'^[\w\-.]*$', self.scope), \
             "ERROR: scope {} does not match scope-pattern! Needs to be \\w or '-'.".format(self.scope)
         # The global scope string defining the exact nested position of this Component in the Graph.
         # e.g. "/core/component1/sub-component-a"
@@ -159,13 +160,20 @@ class Component(Specifiable):
         self.constant_op_records = set()
         # Whether we know already all our API-methods' call args' spaces.
         self.input_complete = False
+        # Short-circuit-set to True, if no variables are generated by this Component anyway.
+        if re.search(r'\spass\n$', inspect.getsource(self.create_variables)) and \
+                re.search(r'\spass\n$', inspect.getsource(self.check_input_spaces)):
+            self.input_complete = True
+
         # Whether all our sub-Components are input-complete. Only after that point, we can run our _variables graph_fn.
         self.variable_complete = False
+        # Has this component been built yet by the graph builder?
+        self.built = False
 
         # All Variables that are held by this component (and its sub-components) by name.
         # key=full-scope variable name (scope=component/sub-component scope)
         # value=the actual variable
-        self.variables = {}
+        self.variable_registry = {}
         # All summary ops that are held by this component (and its sub-components) by name.
         # key=full-scope summary name (scope=component/sub-component scope)
         # value=the actual summary op
@@ -240,15 +248,10 @@ class Component(Specifiable):
         Returns:
             bool: Whether this Component is input-complete or not.
         """
-        assert self.input_complete is False
-
-        self.input_complete = True
-
-        # Short-circuit-set to True, if no variables are generated by this Component anyway.
-        if re.search(r'\spass\n$', inspect.getsource(self.create_variables)) and \
-                re.search(r'\spass\n$', inspect.getsource(self.check_input_spaces)):
+        if self.input_complete is True:
             return True
 
+        self.input_complete = True
         # Loop through all API methods.
         for method_name, api_method_rec in self.api_methods.items():
             # This API method doesn't have to be completed, ignore and don't add it to space_dict.
@@ -256,15 +259,16 @@ class Component(Specifiable):
                 continue
 
             # Loop through all of this API-method's input parameter names and check, whether they
-            # all have a Space  defined.
+            # all have a Space defined.
             for input_name in api_method_rec.input_names:
                 assert input_name in self.api_method_inputs
                 # This one is not defined yet -> Component is not input-complete.
                 if self.api_method_inputs[input_name] is None:
                     self.input_complete = False
+                    self.logger.debug("Found incomplete input_name {} for method {}.".format(input_name, method_name))
                     return False
-                # OR: API-method has a var-positional parameter: Check whether it has been called at least once (in
-                # which case we have Space information stored under "*args[0]").
+                # API-method has a var-positional parameter (*args): Check whether it has been called at
+                # least once (in which case we have Space information stored under "args[0]").
                 elif self.api_method_inputs[input_name] == "*flex":
                     # Check all keys "input_name[n]" for any None. If one None found -> input incomplete.
                     idx = 0
@@ -277,17 +281,33 @@ class Component(Specifiable):
                             # No input defined (has not been called) -> Not input complete.
                             else:
                                 self.input_complete = False
+                                self.logger.debug(
+                                    "Found incomplete flex key {} for method {}.".format(key, method_name))
+
                                 return False
                         elif self.api_method_inputs[key] is None:
                             self.input_complete = False
+                            self.logger.debug(
+                                "Found incomplete flex key {} for method {}.".format(key, method_name))
                             return False
                         idx += 1
+                # API-method has **kwargs parameters: Check whether it has been called at
+                # least once (in which case we have Space information stored under "kwargs['some key']").
+                elif self.api_method_inputs[input_name] == "**flex":
+                    keys = [k for k in self.api_method_inputs if re.match(r'{}\['.format(input_name), k)]
+                    # Check all keys "input_name[n]" for any None. If one None found -> input incomplete.
+                    for key in keys:
+                        if self.api_method_inputs[key] is None:
+                            self.logger.debug(
+                                "Found incomplete kwargs key {} for method {}.".format(key, method_name))
+                            self.input_complete = False
+                            return False
         return True
 
     def check_variable_completeness(self):
         """
         Checks, whether this Component is input-complete AND all our sub-Components are input-complete.
-        At that point, all variables are defined and we can run the `_variables` graph_fn.
+        At that point, all variables are defined and we can run the `variables` graph_fn.
 
         Returns:
             bool: Whether this Component is "variables-complete".
@@ -301,6 +321,8 @@ class Component(Specifiable):
 
         # Simply check all direct sub-Components for variable-completeness.
         for direct_child in self.sub_components.values():
+            if re.search(r'^\.helper-', direct_child.scope):
+                continue
             if not direct_child.check_variable_completeness():
                 return False
         self.variable_complete = True
@@ -358,6 +380,8 @@ class Component(Specifiable):
         # Add all created variables up the parent/container hierarchy.
         self.propagate_variables()
 
+        self.built = True
+
     def check_input_spaces(self, input_spaces, action_space=None):
         """
         Should check on the nature of all in-Sockets Spaces of this Component. This method is called automatically
@@ -406,15 +430,16 @@ class Component(Specifiable):
             # key = re.sub(r'({}).*?([\w\-.]+):\d+$'.format(self.global_scope), r'\1/\2', var.name)
             key = re.sub(r':\d+$', "", var.name)
             # Already registered: Must be the same (shared) variable.
-            if key in self.variables:
-                assert self.variables[key] is var,\
+            if key in self.variable_registry:
+                assert self.variable_registry[key] is var,\
                     "ERROR: Key '{}' in {}.variables already exists, but holds a different variable " \
-                    "({} vs {})!".format(key, self.global_scope, self.variables[key], var)
+                    "({} vs {})!".format(key, self.global_scope, self.variable_registry[key], var)
             # New variable: Register.
             else:
-                self.variables[key] = var
+                self.variable_registry[key] = var
                 # Auto-create the summary for the variable.
-                summary_name = var.name[len(self.global_scope) + (1 if self.global_scope else 0):]
+                scope_to_use = self.reuse_variable_scope or self.global_scope
+                summary_name = var.name[len(scope_to_use) + (1 if scope_to_use else 0):]
                 summary_name = re.sub(r':\d+$', "", summary_name)
                 self.create_summary(summary_name, var)
 
@@ -466,7 +491,6 @@ class Component(Specifiable):
         Returns:
             DataOp: The actual variable (dependent on the backend) or - if from
                 a ContainerSpace - a FlattenedDataOp or ContainerDataOp depending on the Space.
-
         """
 
         # Overwrite the given trainable parameter, iff self.trainable is actually defined as a bool.
@@ -474,12 +498,12 @@ class Component(Specifiable):
 
         # Called as getter.
         if shape is None and initializer is None and from_space is None:
-            if name not in self.variables:
+            if name not in self.variable_registry:
                 raise KeyError(
                     "Variable with name '{}' not found in registry of Component '{}'!".format(name, self.name)
                 )
             # TODO: Maybe try both the pure name AND the name with global-scope in front.
-            return self.variables[name]
+            return self.variable_registry[name]
 
         # Called as setter.
         var = None
@@ -491,8 +515,7 @@ class Component(Specifiable):
                 use_resource
             )
 
-        # TODO: Figure out complete concept for python/numpy based Components (including their handling of variables).
-        # Assume that when using pytorch, we use Python/numpy collections to store data.
+        # TODO: Revise possible arg combinations, move in utils.
         elif self.backend == "python" or get_backend() == "python" or get_backend() == "pytorch":
             if add_batch_rank is not False and isinstance(add_batch_rank, int):
                 if isinstance(add_time_rank, int):
@@ -507,9 +530,20 @@ class Component(Specifiable):
             elif initializer is not None:
                 # Return
                 var = initializer
+            elif shape is not None:
+                # Use python list if possible:
+                if len(shape) == 1:
+                    if dtype == int:
+                        var = [0 for _ in range(shape[0])]
+                    elif dtype == float:
+                        var = [0.0 for _ in range(shape[0])]
+                else:
+                    if dtype == int:
+                        var = np.zeros(shape, dtype=np.int32)
+                    elif dtype == float:
+                        var = np.zeros(shape, dtype=np.float32)
             else:
                 var = []
-            return var
 
         # Direct variable creation (using the backend).
         elif get_backend() == "tf":
@@ -547,9 +581,9 @@ class Component(Specifiable):
         # TODO: What about a var from Tuple space?
         if isinstance(var, OrderedDict):
             for sub_key, v in var.items():
-                self.variables[key + sub_key] = v
+                self.variable_registry[key + sub_key] = v
         else:
-            self.variables[key] = var
+            self.variable_registry[key] = var
 
         return var
 
@@ -641,7 +675,7 @@ class Component(Specifiable):
                 ret = {}
                 for v in collection_variables:
                     lookup = re.sub(r':\d+$', "", v.name)
-                    if lookup in self.variables:
+                    if lookup in self.variable_registry:
                         if global_scope:
                             # Replace the scope separator with a custom one.
                             ret[re.sub(r'(/|{}|{})'.format(FLAT_TUPLE_CLOSE, FLAT_TUPLE_OPEN),
@@ -661,7 +695,7 @@ class Component(Specifiable):
             get_ref = kwargs.pop("get_ref", False)
 
             if len(names) == 0:
-                names = list(self.variables.keys())
+                names = list(self.variable_registry.keys())
             return self.get_variables_by_name(
                 *names, custom_scope_separator=custom_scope_separator, global_scope=global_scope,
                 get_ref=get_ref
@@ -693,35 +727,50 @@ class Component(Specifiable):
         if get_backend() == "tf":
             for name in names:
                 global_scope_name = ((self.global_scope + "/") if self.global_scope else "") + name
-                if name in self.variables:
-                    variables[re.sub(r'/', custom_scope_separator, name)] = self.variables[name]
-                elif global_scope_name in self.variables:
+                if name in self.variable_registry:
+                    variables[re.sub(r'/', custom_scope_separator, name)] = self.variable_registry[name]
+                elif global_scope_name in self.variable_registry:
                     if global_scope:
-                        variables[re.sub(r'/', custom_scope_separator, global_scope_name)] = self.variables[
+                        variables[re.sub(r'/', custom_scope_separator, global_scope_name)] = self.variable_registry[
                             global_scope_name]
                     else:
-                        variables[name] = self.variables[global_scope_name]
+                        variables[name] = self.variable_registry[global_scope_name]
         elif get_backend() == "pytorch":
+            # Unpack tuple.
+            if isinstance(names, tuple) and len(names) == 1:
+                names = names[0]
+            # print("names = ", names)
+            state = None
+            if TraceContext.DEFINE_BY_RUN_CONTEXT == "execution" and hasattr(self, "get_state"):
+                state = self.get_state()
             for name in names:
                 global_scope_name = ((self.global_scope + "/") if self.global_scope else "") + name
-                if name in self.variables:
+                if state is not None:
+                    # Generally using underscores in attribute names, not scope separates
+                    lookup = name.replace("-", "_")
+                    if lookup in state:
+                        variables[name] = state[lookup]
+                    elif name in state:
+                        variables[name] = state[name]
+                elif name in self.variable_registry:
                     if get_ref:
-                        variables[re.sub(r'/', custom_scope_separator, name)] = self.variables[name]
+                        variables[re.sub(r'/', custom_scope_separator, name)] = self.variable_registry[name]
                     else:
-                        variables[re.sub(r'/', custom_scope_separator, name)] = self.read_variable(self.variables[name])
-                elif global_scope_name in self.variables:
+                        variables[re.sub(r'/', custom_scope_separator, name)] = self.read_variable(self.variable_registry[name])
+                elif global_scope_name in self.variable_registry:
                     if global_scope:
                         if get_ref:
                             variables[re.sub(r'/', custom_scope_separator, global_scope_name)] = \
-                                self.variables[global_scope_name]
+                                self.variable_registry[global_scope_name]
                         else:
                             variables[re.sub(r'/', custom_scope_separator, global_scope_name)] = \
-                                self.read_variable(self.variables[global_scope_name])
+                                self.read_variable(self.variable_registry[global_scope_name])
                     else:
                         if get_ref:
-                            variables[name] = self.variables[global_scope_name]
+                            variables[name] = self.variable_registry[global_scope_name]
                         else:
-                            variables[name] = self.read_variable(self.variables[global_scope_name])
+                            variables[name] = self.read_variable(self.variable_registry[global_scope_name])
+
         return variables
 
     def create_summary(self, name, values, summary_type="histogram"):
@@ -870,16 +919,16 @@ class Component(Specifiable):
         return_ = False
         # This is the final-return recursive call-level.
         if list_ is None:
-            list_ = dict()
+            list_ = {}
             return_ = True
         if level_ not in list_:
-            list_[level_] = list()
+            list_[level_] = []
         list_[level_].append(self)
         level_ += 1
         for sub_component in self.sub_components.values():
             sub_component.get_all_sub_components(list_, level_)
         if return_:
-            ret = list()
+            ret = []
             for l in sorted(list_.keys(), reverse=True):
                 ret.extend(sorted(list_[l], key=lambda c: c.scope))
             if exclude_self:
@@ -1022,15 +1071,15 @@ class Component(Specifiable):
             return
 
         # Add all our variables to parent's variable registry.
-        keys = keys or self.variables.keys()
+        keys = keys or self.variable_registry.keys()
         for key in keys:
             # If already there (bubbled up from some child component that was input complete before us)
             # -> Make sure the variable object is identical.
-            if key in self.parent_component.variables:
-                if self.variables[key] is not self.parent_component.variables[key]:
+            if key in self.parent_component.variable_registry:
+                if self.variable_registry[key] is not self.parent_component.variable_registry[key]:
                     raise RLGraphError("ERROR: Variable registry of '{}' already has a variable under key '{}'!". \
                           format(self.parent_component.name, key))
-            self.parent_component.variables[key] = self.variables[key]
+            self.parent_component.variable_registry[key] = self.variable_registry[key]
 
         # Recurse up the container hierarchy.
         self.parent_component.propagate_variables(keys)
@@ -1060,7 +1109,7 @@ class Component(Specifiable):
             Component: The copied component object.
         """
         # Make sure we are still in the assembly phase (should not copy actual ops).
-        assert self.input_complete is False, "ERROR: Cannot copy a Component ('{}') that has already been built!". \
+        assert self.built is False, "ERROR: Cannot copy a Component ('{}') that has already been built!". \
             format(self.name)
 
         if scope is None:
@@ -1134,14 +1183,15 @@ class Component(Specifiable):
             ref.set_value(value)
 
     @staticmethod
-    def read_variable(variable, indices=None, dtype=None):
+    def read_variable(variable, indices=None, dtype=None, shape=None):
         """
         Reads a variable.
 
         Args:
             variable (DataOp): The variable whose value to read.
             indices (Optional[np.ndarray,tf.Tensor]): Indices (if any) to fetch from the variable.
-
+            dtype (Optional[torch.dtype]): Optional dtype to convert read values to.
+            shape (Optional[tuple]): Optional default shape.
         Returns:
             any: Variable values.
         """
@@ -1159,6 +1209,10 @@ class Component(Specifiable):
             # Lists or numpy arrays may be used to store mutable state that does not need
             # tensor operations.
             elif isinstance(variable, list) or isinstance(variable, np.ndarray):
+                if TraceContext.DEFINE_BY_RUN_CONTEXT == "building" \
+                        and shape is not None and (indices is None or len(indices) == 0):
+                    return torch.zeros(shape, dtype=dtype)
+
                 if indices is not None:
                     ret = []
                     for i in indices:
@@ -1174,9 +1228,12 @@ class Component(Specifiable):
                             elif dtype == torch.uint8:
                                 ret.append(val.byte())
                     # Stack list into one Tensor with a btach dim.
-                    return torch.stack(ret)
+                    return torch.stack(ret) if ret else ret
                 else:
                     return variable
+            else:
+                # Catch all for raw types.
+                return variable
 
     def sub_component_by_name(self, scope_name):
         """
@@ -1207,12 +1264,47 @@ class Component(Specifiable):
         """
         Optionally execute post-build calls.
         """
+        # Try resetting state.
+        component_state = self.get_state()
+        if component_state is not None:
+            for name in component_state.keys():
+                if hasattr(self, name) and isinstance(getattr(self, name), (float, int)):
+                    self.__setattr__(name, 0)
+
+    def get_state(self):
+        """
+        Optionally provide define-by-run state as dict.
+
+        Returns:
+            dict: Names and values of variables.
+        """
         pass
 
-    @rlgraph_api(returns=1)
-    def _graph_fn__variables(self):
+    def get_helper_component(self, type_, *args, **kwargs):
         """
-        Outputs all of this Component's variables in a DataOpDict (API-method "_variables").
+        Returns a helper component of the given type (only one helper component per type is allowed
+        and necessary). If a helper of the type does not exist yet in `self`, create a new one.
+
+        Args:
+            type_ (str): The type as a string of the helper Component to return.
+
+        Returns:
+            Component: The helper component.
+        """
+        name = ".helper-"+type_+"-{}".format(uuid.uuid4())
+        helper = self.sub_components.get(name)
+        if helper is None:
+            kwargs.update(dict(type=type_, scope=name))
+            if len(args) > 0:
+                kwargs.update({"_args": args})
+            helper = Component.from_spec(kwargs)
+            self.add_components(helper)
+        return helper
+
+    @rlgraph_api(returns=1, requires_variable_completeness=True)
+    def _graph_fn_variables(self):
+        """
+        Outputs all of this Component's variables in a DataOpDict (API-method "variables").
 
         This can be used e.g. to sync this Component's variables into another Component, which owns
         a Synchronizable() as a sub-component. The returns values of this graph_fn are then sent into
@@ -1226,6 +1318,12 @@ class Component(Specifiable):
         variables_dict = self.get_variables(custom_scope_separator="-")
         return DataOpDict(variables_dict)
 
+    def _variables(self):
+        """
+        OBSOLETED API method. New "variables()" should be used.
+        """
+        raise RLGraphObsoletedError("API-method", "_variables()", "variables()")
+
     @staticmethod
     def reset_profile():
         """
