@@ -20,14 +20,13 @@ from __future__ import print_function
 from collections import deque
 
 import numpy as np
-from six.moves import xrange as range_
-
 from rlgraph import get_backend
+from rlgraph.components.layers.preprocessing import PreprocessLayer
 from rlgraph.spaces.space_utils import sanity_check_space
 from rlgraph.utils.decorators import rlgraph_api
 from rlgraph.utils.ops import FlattenedDataOp, unflatten_op
-from rlgraph.utils.util import get_rank, get_shape, force_list
-from rlgraph.components.layers.preprocessing import PreprocessLayer
+from rlgraph.utils.util import get_rank, force_list
+from six.moves import xrange as range_
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -82,7 +81,7 @@ class Sequence(PreprocessLayer):
             self.deque = deque([], maxlen=self.sequence_length)
 
     def get_preprocessed_space(self, space):
-        ret = dict()
+        ret = {}
         for key, value in space.flatten().items():
             shape = list(value.shape)
             if self.add_rank:
@@ -109,12 +108,9 @@ class Sequence(PreprocessLayer):
     def create_variables(self, input_spaces, action_space=None):
         in_space = input_spaces["preprocessing_inputs"]
         self.output_spaces = self.get_preprocessed_space(in_space)
-
         self.index = self.get_variable(name="index", dtype="int", initializer=-1, trainable=False)
-        if self.backend == "python" or get_backend() == "python" or get_backend() == "pytorch":
-            # TODO get variable does not return an int for python
-            self.index = -1
-        else:
+
+        if get_backend() == "tf":
             self.buffer = self.get_variable(
                 name="buffer", trainable=False, from_space=in_space,
                 add_batch_rank=self.batch_size if in_space.has_batch_rank is not False else False,
@@ -247,9 +243,12 @@ class Sequence(PreprocessLayer):
 
                     # Must pass the sequence through a placeholder_with_default dummy to set back the
                     # batch rank to '?', instead of 1 (1 would confuse the auto Space inference).
-                    sequences[key] = tf.placeholder_with_default(
-                        sequence, shape=(None,) + tuple(get_shape(sequence)[1:])
-                    )
+                    #sequences[key] = tf.placeholder_with_default(
+                    #    sequence, shape=(None,) + tuple(get_shape(sequence)[1:])
+                    #)
+                    sequence._batch_rank = 0
+                    sequences[key] = sequence
+
             # TODO implement transpose
                 return sequences
 
