@@ -19,7 +19,8 @@ from __future__ import print_function
 
 from rlgraph import get_backend
 from rlgraph.components import Component
-from rlgraph.utils.decorators import rlgraph_api
+from rlgraph.spaces import ContainerSpace
+from rlgraph.utils.decorators import rlgraph_api, graph_fn
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -39,6 +40,7 @@ class LossFunction(Component):
         super(LossFunction, self).__init__(scope=kwargs.pop("scope", "loss-function"), **kwargs)
 
         self.discount = discount
+        self.action_space = None
 
     @rlgraph_api
     def loss(self, *inputs):
@@ -85,3 +87,19 @@ class LossFunction(Component):
             return tf.reduce_mean(input_tensor=loss_per_item, axis=0)
         elif get_backend() == "pytorch":
             return torch.mean(loss_per_item, 0)
+
+    @graph_fn(flatten_ops=True)
+    def _graph_fn_average_over_container_keys(self, loss_per_item):
+        """
+        If action space is a container space, average losses for each action.
+        """
+        if get_backend() == "tf":
+            if isinstance(self.action_space, ContainerSpace):
+                loss_per_item = tf.stack(list(loss_per_item.values()))
+                loss_per_item = tf.reduce_mean(loss_per_item, axis=0)
+            return loss_per_item
+        elif get_backend() == "pytorch":
+            if isinstance(self.action_space, ContainerSpace):
+                loss_per_item = torch.stack(list(loss_per_item.values()))
+                loss_per_item = torch.mean(loss_per_item, 0)
+            return loss_per_item
