@@ -22,11 +22,12 @@ import operator
 from six.moves import xrange as range_
 
 from rlgraph import get_backend
-from rlgraph.utils.util import SMALL_NUMBER, get_rank, convert_dtype as dtype_
+from rlgraph.utils import util, DataOpDict
+from rlgraph.utils.execution_util import define_by_run_unflatten
+from rlgraph.utils.util import SMALL_NUMBER, get_rank
 from rlgraph.components.memories.memory import Memory
 from rlgraph.components.helpers.mem_segment_tree import MemSegmentTree, MinSumSegmentTree
 from rlgraph.utils.decorators import rlgraph_api
-from rlgraph.spaces import Dict
 
 if get_backend() == "pytorch":
     import torch
@@ -49,19 +50,14 @@ class MemPrioritizedReplay(Memory):
         self.size = 0
         self.max_priority = 1.0
         self.alpha = alpha
-
         self.beta = beta
         self.next_states = next_states
 
         self.default_new_weight = np.power(self.max_priority, self.alpha)
 
     def create_variables(self, input_spaces, action_space=None):
-        # Store our record-space for convenience.
-        self.record_space = input_spaces["records"]
-        self.record_space_flat = Dict(self.record_space.flatten(
-            custom_scope_separator="/", scope_separator_at_start=False), add_batch_rank=True)
+        super(MemPrioritizedReplay, self).create_variables(input_spaces, action_space)
         self.priority_capacity = 1
-
         while self.priority_capacity < self.capacity:
             self.priority_capacity *= 2
 
@@ -77,48 +73,11 @@ class MemPrioritizedReplay(Memory):
             capacity=self.priority_capacity
         )
 
-    def _read_records(self, indices):
-        """
-        Obtains record values for the provided indices.
-
-        Args:
-            indices ndarray: Indices to read. Assumed to be not contiguous.
-
-        Returns:
-             dict: Record value dict.
-        """
-        records = {}
-        for name in self.record_space_flat.keys():
-            records[name] = []
-
-        if self.size > 0:
-            for index in indices:
-                record = self.memory_values[index]
-                for name in self.record_space_flat.keys():
-                    records[name].append(record[name])
-
-        else:
-            # TODO figure out how to do default handling in pytorch builds.
-            # Fill with default vals for build.
-            for name in self.record_space_flat.keys():
-                if get_backend() == "pytorch":
-                    records[name] = torch.zeros(self.record_space_flat[name].shape,
-                                                dtype=dtype_(self.record_space_flat[name].dtype, "pytorch"))
-                else:
-                    records[name] = np.zeros(self.record_space_flat[name].shape)
-
-        # Convert if necessary: list of tensors fails at space inference otherwise.
-        if get_backend() == "pytorch":
-            for name in self.record_space_flat.keys():
-                records[name] = torch.squeeze(torch.stack(records[name]))
-
-        return records
-
     @rlgraph_api(flatten_ops=True)
     def _graph_fn_insert_records(self, records):
-        if records is None or get_rank(records['rewards']) == 0:
+        if records is None or get_rank(records[self.terminal_key]) == 0:
             return
-        num_records = len(records['rewards'])
+        num_records = len(records[self.terminal_key])
 
         if num_records == 1:
             if self.index >= self.size:
@@ -131,7 +90,7 @@ class MemPrioritizedReplay(Memory):
             i = 0
             for insert_index in insert_indices:
                 self.merged_segment_tree.insert(insert_index, self.default_new_weight)
-                record = dict()
+                record = {}
                 for name, record_values in records.items():
                     record[name] = record_values[i]
                 if insert_index >= self.size:
@@ -146,9 +105,10 @@ class MemPrioritizedReplay(Memory):
 
     @rlgraph_api
     def _graph_fn_get_records(self, num_records=1):
+        available_records = min(num_records, self.size)
         indices = []
         prob_sum = self.merged_segment_tree.sum_segment_tree.get_sum(0, self.size - 1)
-        samples = np.random.random(size=(num_records,)) * prob_sum
+        samples = np.random.random(size=(available_records,)) * prob_sum
         for sample in samples:
             indices.append(self.merged_segment_tree.sum_segment_tree.index_of_prefixsum(prefix_sum=sample))
 
@@ -167,13 +127,24 @@ class MemPrioritizedReplay(Memory):
         else:
             indices = np.asarray(indices)
             weights = np.asarray(weights)
-        return self._read_records(indices=indices), indices, weights
+
+        records = DataOpDict()
+        for name, variable in self.memory.items():
+            records[name] = self.read_variable(variable, indices, dtype=
+            util.convert_dtype(self.flat_record_space[name].dtype, to="pytorch"))
+        records = define_by_run_unflatten(records)
+        return records, indices, weights
 
     @rlgraph_api(must_be_complete=False)
     def _graph_fn_update_records(self, indices, update):
-        if len(indices) > 0 and indices[0]:
-            for index, loss in zip(indices, update):
-                priority = np.power(loss, self.alpha)
-                self.merged_segment_tree.insert(index, priority)
-                self.max_priority = max(self.max_priority, priority)
+        for index, loss in zip(indices, update):
+            priority = np.power(loss, self.alpha)
+            self.merged_segment_tree.insert(index, priority)
+            self.max_priority = max(self.max_priority, priority)
 
+    def get_state(self):
+        return {
+            "size": self.size,
+            "index": self.index,
+            "max_priority": self.max_priority
+        }
\ No newline at end of file
