@@ -17,11 +17,10 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from collections import OrderedDict
-
 from rlgraph import get_backend
 from rlgraph.components.memories.memory import Memory
-from rlgraph.utils import util
+from rlgraph.utils import util, DataOpDict
+from rlgraph.utils.execution_util import define_by_run_unflatten
 from rlgraph.utils.util import get_batch_size
 from rlgraph.utils.decorators import rlgraph_api
 
@@ -44,40 +43,28 @@ class RingBuffer(Memory):
         super(RingBuffer, self).__init__(capacity, scope=scope, **kwargs)
 
         self.index = None
-        self.size = None
         self.states = None
         self.num_episodes = None
         self.episode_indices = None
+        self.flat_record_space = None
 
     def create_variables(self, input_spaces, action_space=None):
         super(RingBuffer, self).create_variables(input_spaces, action_space)
 
         # Record space must contain 'terminals' for a ring buffer memory.
         assert 'terminals' in self.record_space
+        self.index = self.get_variable(name="index", dtype=int, trainable=False, initializer=0)
+        # Num episodes present.
+        self.num_episodes = self.get_variable(name="num-episodes", dtype=int, trainable=False, initializer=0)
 
-        # Main buffer index.
-        if get_backend() == "tf":
-            self.index = self.get_variable(name="index", dtype=int, trainable=False, initializer=0)
-            # Number of elements present.
-            self.size = self.get_variable(name="size", dtype=int, trainable=False, initializer=0)
-
-            # Num episodes present.
-            self.num_episodes = self.get_variable(name="num-episodes", dtype=int, trainable=False, initializer=0)
-
-            # Terminal indices contiguously arranged.
-            self.episode_indices = self.get_variable(name="episode-indices", shape=(self.capacity,),
-                                                     dtype=int, trainable=False)
-        elif get_backend() == "pytorch":
-            self.index = 0
-            self.size = 0
-            self.num_episodes = 0
-            self.episode_indices = [0] * self.capacity
-            self.record_registry["terminals"] = [0 for _ in self.record_registry["terminals"]]
+        # Terminal indices contiguously arranged.
+        self.episode_indices = self.get_variable(name="episode-indices", shape=(self.capacity,),
+                                                 dtype=int, trainable=False)
 
     @rlgraph_api(flatten_ops=True)
     def _graph_fn_insert_records(self, records):
         if get_backend() == "tf":
-            num_records = get_batch_size(records["terminals"])
+            num_records = get_batch_size(records[self.terminal_key])
             index = self.read_variable(self.index)
 
             # Episodes before inserting these records.
@@ -86,20 +73,22 @@ class RingBuffer(Memory):
 
             # Episodes previously existing in the range we inserted to as indicated
             # by count of terminals in the that slice.
-            insert_terminal_slice = self.read_variable(self.record_registry['terminals'], update_indices)
+            insert_terminal_slice = self.read_variable(self.memory[self.terminal_key], update_indices)
 
             # Shift episode indices.
             with tf.control_dependencies([update_indices, index, prev_num_episodes, insert_terminal_slice]):
                 index_updates = []
 
                 # Newly inserted episodes.
-                inserted_episodes = tf.reduce_sum(input_tensor=tf.cast(records['terminals'], dtype=tf.int32), axis=0)
+                inserted_episodes = tf.reduce_sum(input_tensor=tf.cast(records[self.terminal_key], dtype=tf.int32), axis=0)
                 episodes_in_insert_range = tf.reduce_sum(
                     input_tensor=tf.cast(insert_terminal_slice, dtype=tf.int32), axis=0
                 )
                 num_episode_update = prev_num_episodes - episodes_in_insert_range + inserted_episodes
 
                 # Shift contiguous episode indices.
+                # prev_num_episodes = tf.Print(prev_num_episodes, [inserted_episodes, episodes_in_insert_range], summarize=100,
+                #                              message="inserted_episodes, episodes in range = ")
                 index_updates.append(self.assign_variable(
                         ref=self.episode_indices[:prev_num_episodes - episodes_in_insert_range],
                         value=self.episode_indices[episodes_in_insert_range:prev_num_episodes]
@@ -109,15 +98,15 @@ class RingBuffer(Memory):
                 # ending at previous count minus removed + inserted.
                 slice_start = prev_num_episodes - episodes_in_insert_range
                 slice_end = num_episode_update
-                # update_indices = tf.Print(update_indices, [update_indices, tf.shape(update_indices)],
-                #                           summarize=100, message='\n update indices / shape = ')
 
             # Update indices and size.
             with tf.control_dependencies(index_updates):
                 index_updates = []
 
                 # Actually update indices.
-                mask = tf.boolean_mask(tensor=update_indices, mask=records['terminals'])
+                mask = tf.boolean_mask(tensor=update_indices, mask=records[self.terminal_key])
+                # mask = tf.Print(mask, [self.episode_indices, update_indices, mask, slice_start, slice_end],
+                #                 summarize=100, message="update, mask, start, end")
                 index_updates.append(self.assign_variable(
                     ref=self.episode_indices[slice_start:slice_end],
                     value=mask
@@ -133,9 +122,9 @@ class RingBuffer(Memory):
             # Updates all the necessary sub-variables in the record.
             with tf.control_dependencies(index_updates):
                 record_updates = []
-                for key in self.record_registry:
+                for key in self.memory:
                     record_updates.append(self.scatter_update_variable(
-                        variable=self.record_registry[key],
+                        variable=self.memory[key],
                         indices=update_indices,
                         updates=records[key]
                     ))
@@ -145,18 +134,18 @@ class RingBuffer(Memory):
                 return tf.no_op()
         elif get_backend() == "pytorch":
             # TODO: Unclear if we should do this in numpy and then convert to torch once we sample.
-            num_records = get_batch_size(records["terminals"])
+            num_records = get_batch_size(records[self.terminal_key])
             update_indices = torch.arange(self.index, self.index + num_records) % self.capacity
 
             # Newly inserted episodes.
-            inserted_episodes = torch.sum(records['terminals'].int(), 0)
+            inserted_episodes = torch.sum(records[self.terminal_key].int(), 0)
 
             # Episodes previously existing in the range we inserted to as indicated
             # by count of terminals in the that slice.
             episodes_in_insert_range = 0
             # Count terminals in inserted range.
             for index in update_indices:
-                episodes_in_insert_range += int(self.record_registry["terminals"][index])
+                episodes_in_insert_range += int(self.memory[self.terminal_key][index])
             num_episode_update = self.num_episodes - episodes_in_insert_range + inserted_episodes
             self.episode_indices[:self.num_episodes - episodes_in_insert_range] = \
                 self.episode_indices[episodes_in_insert_range:self.num_episodes]
@@ -166,7 +155,7 @@ class RingBuffer(Memory):
             slice_start = self.num_episodes - episodes_in_insert_range
             slice_end = num_episode_update
 
-            byte_terminals = records["terminals"].byte()
+            byte_terminals = records[self.terminal_key].byte()
             mask = torch.masked_select(update_indices, byte_terminals)
             self.episode_indices[slice_start:slice_end] = mask
 
@@ -176,9 +165,9 @@ class RingBuffer(Memory):
             self.size = min(self.size + num_records, self.capacity)
 
             # Updates all the necessary sub-variables in the record.
-            for key in self.record_registry:
+            for key in self.memory:
                 for i, val in zip(update_indices, records[key]):
-                    self.record_registry[key][i] = val
+                    self.memory[key][i] = val
 
             # The TF version returns no-op, return None so return-val inference system does not throw error.
             return None
@@ -186,15 +175,22 @@ class RingBuffer(Memory):
     @rlgraph_api
     def _graph_fn_get_records(self, num_records=1):
         if get_backend() == "tf":
+            stored_records = self.read_variable(self.size)
+            available_records = tf.minimum(x=num_records, y=stored_records)
             index = self.read_variable(self.index)
-            indices = tf.range(start=index - num_records, limit=index) % self.capacity
+            indices = tf.range(start=index - available_records, limit=index) % self.capacity
             return self._read_records(indices=indices)
         elif get_backend() == "pytorch":
-            indices = np.arange(self.index - num_records, self.index) % self.capacity
-            records = OrderedDict()
-            for name, variable in self.record_registry.items():
-                records[name] = self.read_variable(variable, indices,
-                                                   dtype=util.convert_dtype(self.record_space[name].dtype, to="pytorch"))
+            available_records = min(num_records, self.size)
+            indices = np.arange(self.index - available_records, self.index) % self.capacity
+            records = DataOpDict()
+
+            for name, variable in self.memory.items():
+                records[name] = self.read_variable(variable, indices, dtype=
+                                                   util.convert_dtype(self.flat_record_space[name].dtype, to="pytorch"),
+                                                   shape=self.flat_record_space[name].shape)
+
+            records = define_by_run_unflatten(records)
             return records
 
     @rlgraph_api(ok_to_overwrite=True)
@@ -218,9 +214,10 @@ class RingBuffer(Memory):
             )
             # End index is just the pointer to the most recent episode.
             limit = self.episode_indices[stored_episodes - 1]
-            limit += tf.where(condition=(start < limit), x=0, y=self.capacity)
 
-            indices = tf.range(start=start, limit=limit) % self.capacity
+            limit += tf.where(condition=(start < limit), x=0, y=self.capacity - 1)
+            # limit = tf.Print(limit, [stored_episodes, start, limit], summarize=100, message="start | limit")
+            indices = tf.range(start=start, limit=limit + 1) % self.capacity
             return self._read_records(indices=indices)
         elif get_backend() == "pytorch":
             stored_episodes = self.num_episodes
@@ -234,11 +231,22 @@ class RingBuffer(Memory):
             # End index is just the pointer to the most recent episode.
             limit = self.episode_indices[stored_episodes - 1]
             if start >= limit:
-                limit += self.capacity
-            indices = torch.arange(start, limit) % self.capacity
+                limit += self.capacity - 1
+            indices = torch.arange(start, limit + 1) % self.capacity
 
-            records = OrderedDict()
-            for name, variable in self.record_registry.items():
-                records[name] = self.read_variable(variable, indices,
-                                                   dtype=util.convert_dtype(self.record_space[name].dtype, to="pytorch"))
+            records = DataOpDict()
+            for name, variable in self.memory.items():
+                records[name] = self.read_variable(variable, indices, dtype=
+                                                   util.convert_dtype(self.flat_record_space[name].dtype, to="pytorch"),
+                                                   shape=self.flat_record_space[name].shape)
+            records = define_by_run_unflatten(records)
             return records
+
+    def get_state(self):
+        return {
+            "index": self.index,
+            "size": self.size,
+            "num_episodes": self.num_episodes,
+            "episode_indices": self.episode_indices,
+            "memory": self.memory
+        }
