@@ -20,15 +20,18 @@ from __future__ import print_function
 import numpy as np
 
 from rlgraph import get_backend
-from rlgraph.utils.rlgraph_errors import RLGraphError
-from rlgraph.spaces import IntBox, FloatBox
-from rlgraph.components.component import Component
-from rlgraph.components.distributions import Normal, Categorical, Beta
-from rlgraph.components.neural_networks.neural_network import NeuralNetwork
 from rlgraph.components.action_adapters.action_adapter import ActionAdapter
-from rlgraph.spaces.space import Space
+from rlgraph.components.action_adapters.action_adapter_utils import get_action_adapter_type_from_distribution_type, \
+    get_distribution_spec_from_action_adapter
+from rlgraph.components.component import Component
+from rlgraph.components.distributions import Distribution
+from rlgraph.components.neural_networks.neural_network import NeuralNetwork
+from rlgraph.spaces import Space, BoolBox, IntBox
+from rlgraph.spaces.space_utils import get_default_distribution_from_space
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
-from rlgraph.utils.ops import FlattenedDataOp
+from rlgraph.utils.execution_util import define_by_run_unflatten
+from rlgraph.utils.ops import FlattenedDataOp, DataOpDict, ContainerDataOp, flat_key_lookup, unflatten_op
+from rlgraph.utils.rlgraph_errors import RLGraphError
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -41,7 +44,7 @@ class Policy(Component):
     A Policy is a wrapper Component that contains a NeuralNetwork, an ActionAdapter and a Distribution Component.
     """
     def __init__(self, network_spec, action_space=None, action_adapter_spec=None,
-                 deterministic=True, scope="policy", **kwargs):
+                 deterministic=True, scope="policy", distributions_spec=None, **kwargs):
         """
         Args:
             network_spec (Union[NeuralNetwork,dict]): The NeuralNetwork Component or a specification dict to build
@@ -55,73 +58,81 @@ class Policy(Component):
             deterministic (bool): Whether to pick actions according to the max-likelihood value or via sampling.
                 Default: True.
 
+            distributions_spec (dict): Specifies bounded and discrete distribution types, and optionally additional
+                configuration parameters such as temperature.
+
             batch_apply (bool): Whether to wrap both the NN and the ActionAdapter with a BatchApply Component in order
                 to fold time rank into batch rank before a forward pass.
         """
         super(Policy, self).__init__(scope=scope, **kwargs)
 
         self.neural_network = NeuralNetwork.from_spec(network_spec)  # type: NeuralNetwork
-
-        # Create the necessary action adapters for this Policy. One for each action space component.
-        self.action_adapters = dict()
-        if action_space is None:
-            self.action_adapters[""] = ActionAdapter.from_spec(action_adapter_spec)
-            self.action_space = self.action_adapters[""].action_space
-            # Assert single component action space.
-            assert len(self.action_space.flatten()) == 1,\
-                "ERROR: Action space must not be ContainerSpace if no `action_space` is given in Policy c'tor!"
-        else:
-            self.action_space = Space.from_spec(action_space)
-            for i, (flat_key, action_component) in enumerate(self.action_space.flatten().items()):
-                if action_adapter_spec is not None:
-                    aa_spec = action_adapter_spec.get(flat_key, action_adapter_spec)
-                    aa_spec["action_space"] = action_component
-                else:
-                    aa_spec = dict(action_space=action_component)
-                self.action_adapters[flat_key] = ActionAdapter.from_spec(aa_spec, scope="action-adapter-{}".format(i))
-
         self.deterministic = deterministic
+        self.action_adapters = {}
+        self.distributions = {}
 
-        # Check for bounded FloatBoxes.
-        self.bounded_action_space = dict()
-        for i, (flat_key, action_component) in enumerate(self.action_space.flatten().items()):
-            if isinstance(action_component, FloatBox):
-                # Unbounded.
-                if action_component.low == float("-inf") and action_component.high == float("inf"):
-                    self.bounded_action_space[flat_key] = False
-                # Bounded.
-                elif action_component.low != float("-inf") and action_component.high != float("inf"):
-                    self.bounded_action_space[flat_key] = True
-                # TODO: Semi-bounded -> Exponential distribution.
-                else:
-                    raise RLGraphError(
-                        "Semi-bounded action spaces are not supported yet! You passed in low={} high={}.".\
-                        format(action_component.low, action_component.high)
-                    )
-            else:
-                self.bounded_action_space[flat_key] = None
+        self.distributions_spec = distributions_spec if distributions_spec is not None else {}
+        self.bounded_distribution_type = self.distributions_spec.get("bounded_distribution_type", "beta")
+        self.discrete_distribution_type = self.distributions_spec.get("discrete_distribution_type", "categorical")
+        # For discrete approximations.
+        self.gumbel_softmax_temperature = self.distributions_spec.get("gumbel_softmax_temperature", 1.0)
 
-        # Figure out our Distributions.
-        self.distributions = dict()
-        for i, (flat_key, action_component) in enumerate(self.action_space.flatten().items()):
-            if isinstance(action_component, IntBox):
-                self.distributions[flat_key] = Categorical(scope="categorical-{}".format(i))
-            # Continuous action space -> Normal distribution (each action needs mean and variance from network).
-            elif isinstance(action_component, FloatBox):
-                # Unbounded -> Normal distribution.
-                if self.bounded_action_space is False:
-                    self.distributions[flat_key] = Normal(scope="normal-{}".format(i))
-                # Bounded -> Beta distribution.
-                else:
-                    self.distributions[flat_key] = Beta(scope="beta-{}".format(i))
-            else:
-                raise RLGraphError("ERROR: `action_component` is of type {} and not allowed in {} Component!".
-                                   format(type(action_space).__name__, self.name))
+        self._create_action_adapters_and_distributions(
+            action_space=action_space, action_adapter_spec=action_adapter_spec
+        )
 
         self.add_components(
             *[self.neural_network] + list(self.action_adapters.values()) + list(self.distributions.values())
         )
 
+    def _create_action_adapters_and_distributions(self, action_space, action_adapter_spec):
+        if action_space is None:
+            adapter = ActionAdapter.from_spec(action_adapter_spec)
+            self.action_space = adapter.action_space
+            # Assert single component action space.
+            assert len(self.action_space.flatten()) == 1, \
+                "ERROR: Action space must not be ContainerSpace if no `action_space` is given in Policy constructor!"
+        else:
+            self.action_space = Space.from_spec(action_space)
+
+        # Figure out our Distributions.
+        for i, (flat_key, action_component) in enumerate(self.action_space.flatten().items()):
+            # Spec dict.
+            if isinstance(action_adapter_spec, dict):
+                aa_spec = flat_key_lookup(action_adapter_spec, flat_key, action_adapter_spec)
+                aa_spec["action_space"] = action_component
+            # Simple type spec.
+            elif not isinstance(action_adapter_spec, ActionAdapter):
+                aa_spec = dict(action_space=action_component)
+            # Direct object.
+            else:
+                aa_spec = action_adapter_spec
+
+            if isinstance(aa_spec, dict) and "type" not in aa_spec:
+                dist_spec = get_default_distribution_from_space(
+                    action_component, self.bounded_distribution_type, self.discrete_distribution_type,
+                    self.gumbel_softmax_temperature
+                )
+
+                self.distributions[flat_key] = Distribution.from_spec(
+                    dist_spec, scope="{}-{}".format(dist_spec["type"], i)
+                )
+                if self.distributions[flat_key] is None:
+                    raise RLGraphError(
+                        "ERROR: `action_component` is of type {} and not allowed in {} Component!".
+                            format(type(action_space).__name__, self.name)
+                    )
+                aa_spec["type"] = get_action_adapter_type_from_distribution_type(
+                    type(self.distributions[flat_key]).__name__
+                )
+                self.action_adapters[flat_key] = ActionAdapter.from_spec(aa_spec, scope="action-adapter-{}".format(i))
+            else:
+                self.action_adapters[flat_key] = ActionAdapter.from_spec(aa_spec, scope="action-adapter-{}".format(i))
+                dist_spec = get_distribution_spec_from_action_adapter(self.action_adapters[flat_key])
+                self.distributions[flat_key] = Distribution.from_spec(
+                    dist_spec, scope="{}-{}".format(dist_spec["type"], i)
+                )
+
     # Define our interface.
     @rlgraph_api
     def get_nn_output(self, nn_input, internal_states=None):
@@ -222,52 +233,32 @@ class Policy(Component):
         out = self.get_logits_parameters_log_probs(nn_input, internal_states)
         action = self._graph_fn_get_action_components(out["logits"], out["parameters"], deterministic)
 
-        return dict(action=action, last_internal_states=out["last_internal_states"])
+        return dict(action=action, last_internal_states=out["last_internal_states"], logits=out["logits"],
+                    parameters=out["parameters"], log_probs=out["log_probs"])
 
     @rlgraph_api
-    def get_action_from_logits_and_parameters(self, logits, parameters, deterministic=None):
+    def get_action_and_log_prob(self, nn_input, internal_states=None, deterministic=None):
         """
-        Returns an action based on NN output, action adapter output and distribution sampling.
-
         Args:
-            logits (any): The `logits` output from `self.get_logits_probabilities_log_probs`.
-            parameters (any): The `parameters` output from `self.get_logits_parameters_log_probs`. Not
-                really needed if action_space is all discrete.
+            nn_input (any): The input to our neural network.
+            internal_states (Optional[any]): The initial internal states going into an RNN-based neural network.
             deterministic (Optional[bool]): If not None, use this to determine whether actions should be drawn
                 from the distribution in max-likelihood (deterministic) or stochastic fashion.
-
         Returns:
-            any: The drawn action.
+            dict:
+                `action`: The drawn action.
+                `log_prob`: The log likelihood of the drawn action.
+                `last_internal_states`: The last internal states (if NN is RNN based, otherwise: None).
         """
         deterministic = self.deterministic if deterministic is None else deterministic
+        out = self.get_logits_parameters_log_probs(nn_input, internal_states)
+        action, log_prob = self._graph_fn_get_action_and_log_prob(out["parameters"], deterministic)
 
-        action = self._graph_fn_get_action_components(logits, parameters, deterministic)
-
-        return dict(action=action)
-
-    @rlgraph_api
-    def get_action_from_logits_and_probabilities(self, logits, probabilities, deterministic=None):
-        """
-        Returns an action based on NN output, action adapter output and distribution sampling.
-
-        Args:
-            logits (any): The `logits` output from `self.get_logits_probabilities_log_probs`.
-            probabilities (any): The `probabilities` output from `self.get_logits_probabilities_log_probs`. Not
-                really needed if action_space is all discrete.
-            deterministic (Optional[bool]): If not None, use this to determine whether actions should be drawn
-                from the distribution in max-likelihood (deterministic) or stochastic fashion.
-
-        Returns:
-            any: The drawn action.
-        """
-        self.logger.warn("Deprecated API method `get_action_from_logits_and_probabilities` used!"
-                         "Use `get_action_from_logits_and_parameters` instead.")
-
-        deterministic = self.deterministic if deterministic is None else deterministic
-
-        action = self._graph_fn_get_action_components(logits, probabilities, deterministic)
-
-        return dict(action=action)
+        return dict(
+            action=action,
+            log_prob=log_prob,
+            last_internal_states=out["last_internal_states"]
+        )
 
     @rlgraph_api
     def get_action_log_probs(self, nn_input, actions, internal_states=None):
@@ -348,7 +339,8 @@ class Policy(Component):
             FlattenedDataOp: A DataOpDict with the different action adapter outputs (keys correspond to
                 structure of `self.action_space`).
         """
-        nn_input = next(iter(nn_input.values()))
+        if isinstance(nn_input, FlattenedDataOp):
+            nn_input = next(iter(nn_input.values()))
 
         ret = FlattenedDataOp()
         for flat_key, action_adapter in self.action_adapters.items():
@@ -386,39 +378,33 @@ class Policy(Component):
 
         return logits, parameters, log_probs
 
-    #@graph_fn(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
-    #def _graph_fn_get_distribution_outputs(self, key, parameters, deterministic):
-    #    """
-    #    Pushes the given `probabilities` through all our distributions' `draw` API-methods and returns a DataOpDict with
-    #    the keys corresponding to our `action_space`.
-    #
-    #    Args:
-    #        parameters (DataOp): The parameters to define a distribution.
-    #        deterministic (DataOp): Passed on to the distributions. Whether to sample deterministically or not.
-    #
-    #    Returns:
-    #        FlattenedDataOp: A DataOpDict with the different distributions' `draw` outputs. Keys always correspond to
-    #            structure of `self.action_space`.
-    #    """
-    #    return self.distributions[key].draw(parameters, deterministic)
-
-    @graph_fn(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
-    def _graph_fn_get_distribution_entropies(self, key, parameters):
+    @graph_fn
+    def _graph_fn_get_distribution_entropies(self, parameters):
         """
         Pushes the given `probabilities` through all our distributions' `entropy` API-methods and returns a
         DataOpDict with the keys corresponding to our `action_space`.
 
         Args:
-            parameters (DataOp): The parameters to define a distribution.
+            parameters (DataOp): The parameters to define a distribution. This could be a ContainerDataOp, which
+                container the parameter pieces for each action component.
 
         Returns:
             FlattenedDataOp: A DataOpDict with the different distributions' `entropy` outputs. Keys always correspond to
                 structure of `self.action_space`.
         """
-        return self.distributions[key].entropy(parameters)
+        ret = FlattenedDataOp()
+        for flat_key, d in self.distributions.items():
+            if flat_key == "":
+                if isinstance(parameters, FlattenedDataOp):
+                    return d.entropy(parameters[flat_key])
+                else:
+                    return d.entropy(parameters)
+            else:
+                ret[flat_key] = d.entropy(parameters.flat_key_lookup(flat_key))
+        return ret
 
-    @graph_fn(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
-    def _graph_fn_get_distribution_log_probs(self, key, parameters, actions):
+    @graph_fn
+    def _graph_fn_get_distribution_log_probs(self, parameters, actions):
         """
         Pushes the given `probabilities` and actions through all our distributions' `log_prob` API-methods and returns a
         DataOpDict with the keys corresponding to our `action_space`.
@@ -431,29 +417,107 @@ class Policy(Component):
             FlattenedDataOp: A DataOpDict with the different distributions' `log_prob` outputs. Keys always correspond
                 to structure of `self.action_space`.
         """
-        # For bounded continuous action spaces, need to unscale (0.0 to 1.0 for beta distribution).
-        if self.bounded_action_space[key] is True:
-            actions = (actions - self.action_space.low) / (self.action_space.high - self.action_space.low)
-        return self.distributions[key].log_prob(parameters, actions)
+        ret = FlattenedDataOp()
+        for flat_key, action_space_component in self.action_space.flatten().items():
+            if flat_key == "":
+                if isinstance(parameters, FlattenedDataOp):
+                    return self.distributions[flat_key].log_prob(parameters[flat_key], actions)
+                else:
+                    return self.distributions[flat_key].log_prob(parameters, actions)
+            else:
+                ret[flat_key] = self.distributions[flat_key].log_prob(
+                    parameters.flat_key_lookup(flat_key), actions.flat_key_lookup(flat_key)
+                )
+        return ret
 
-    @graph_fn(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
-    def _graph_fn_get_action_components(self, key, logits, parameters, deterministic):
-        flat_action_space = self.action_space.flatten()
-        action_space_component = flat_action_space[key]
+    @graph_fn
+    def _graph_fn_get_action_components(self, logits, parameters, deterministic):
+        ret = {}
 
-        # Skip our distribution, iff discrete action-space and deterministic acting (greedy).
-        # In that case, one does not need to create a distribution in the graph each act (only to get the argmax
-        # over the logits, which is the same as the argmax over the probabilities (or log-probabilities)).
-        if isinstance(action_space_component, IntBox) and \
-                (deterministic is True or (isinstance(deterministic, np.ndarray) and deterministic)):
-            actions = self._graph_fn_get_deterministic_action_wo_distribution(logits)
+        # TODO Clean up the checks in here wrt define-by-run processing.
+        for flat_key, action_space_component in self.action_space.flatten().items():
+            # Skip our distribution, iff discrete action-space and deterministic acting (greedy).
+            # In that case, one does not need to create a distribution in the graph each act (only to get the argmax
+            # over the logits, which is the same as the argmax over the probabilities (or log-probabilities)).
+            if isinstance(action_space_component, IntBox) and \
+                    (deterministic is True or (isinstance(deterministic, np.ndarray) and deterministic)):
+                if flat_key == "":
+                    return self._graph_fn_get_deterministic_action_wo_distribution(logits)
+                else:
+                    ret[flat_key] = self._graph_fn_get_deterministic_action_wo_distribution(
+                        logits.flat_key_lookup(flat_key)
+                    )
+            elif isinstance(action_space_component, BoolBox) and \
+                    (deterministic is True or (isinstance(deterministic, np.ndarray) and deterministic)):
+                if get_backend() == "tf":
+                    if flat_key == "":
+                        return tf.greater(logits, 0.5)
+                    else:
+                        ret[flat_key] = tf.greater(logits.flat_key_lookup(flat_key), 0.5)
+                elif get_backend() == "pytorch":
+                    if flat_key == "":
+                        return torch.gt(logits, 0.5)
+                    else:
+                        ret[flat_key] = torch.gt(logits.flat_key_lookup(flat_key), 0.5)
+            else:
+                if flat_key == "":
+                    # Still wrapped as FlattenedDataOp.
+                    if isinstance(parameters, FlattenedDataOp):
+                        return self.distributions[flat_key].draw(parameters[flat_key], deterministic)
+                    else:
+                        return self.distributions[flat_key].draw(parameters, deterministic)
+
+                if isinstance(parameters, ContainerDataOp) and not \
+                        (isinstance(parameters, DataOpDict) and flat_key in parameters):
+                    ret[flat_key] = self.distributions[flat_key].draw(parameters.flat_key_lookup(flat_key), deterministic)
+                else:
+                    ret[flat_key] = self.distributions[flat_key].draw(parameters[flat_key], deterministic)
+
+        if get_backend() == "tf":
+            return unflatten_op(ret)
+        elif get_backend() == "pytorch":
+            return define_by_run_unflatten(ret)
+
+    @graph_fn(returns=2)
+    def _graph_fn_get_action_and_log_prob(self, parameters, deterministic):
+        action = FlattenedDataOp()
+        log_prob = FlattenedDataOp()
+        for flat_key, action_space_component in self.action_space.flatten().items():
+            # Skip our distribution, iff discrete action-space and deterministic acting (greedy).
+            # In that case, one does not need to create a distribution in the graph each act (only to get the argmax
+            # over the logits, which is the same as the argmax over the probabilities (or log-probabilities)).
+            if flat_key == "":
+                if isinstance(parameters, FlattenedDataOp):
+                    params = parameters[""]
+                else:
+                    params = parameters
+            else:
+                params = parameters.flat_key_lookup(flat_key)
+
+            if isinstance(action_space_component, IntBox) and \
+                    (deterministic is True or (isinstance(deterministic, np.ndarray) and deterministic)):
+                action[flat_key] = self._graph_fn_get_deterministic_action_wo_distribution(params)
+                if get_backend() == "tf":
+                    log_prob[flat_key] = tf.reduce_max(params, axis=-1)
+                elif get_backend() == "pytorch":
+                    log_prob[flat_key] = torch.max(params, dim=-1)[0]
+            elif isinstance(action_space_component, BoolBox) and \
+                    (deterministic is True or (isinstance(deterministic, np.ndarray) and deterministic)):
+                if get_backend() == "tf":
+                    action[flat_key] = tf.greater(params, 0.5)
+                elif get_backend() == "pytorch":
+                    action[flat_key] = torch.gt(params, 0.5)
+
+                log_prob[flat_key] = params
+            else:
+                action[flat_key], log_prob[flat_key] = self.distributions[flat_key].sample_and_log_prob(
+                    params, deterministic
+                )
+
+        if len(action) == 1 and "" in action:
+            return action[""], log_prob[""]
         else:
-            actions = self.distributions[key].draw(parameters, deterministic)
-            # If a bounded space (Beta distribution output between 0.0 and 1.0) -> scale correctly.
-            if self.bounded_action_space[key] is True:
-                actions = actions * (self.action_space.high - self.action_space.low) + self.action_space.low
-
-        return actions
+            return action, log_prob
 
     @graph_fn(flatten_ops=True, split_ops=True)
     def _graph_fn_get_deterministic_action_wo_distribution(self, logits):
