@@ -17,8 +17,9 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-import numpy as np
 import random
+
+from rlgraph.environments import Environment
 from six.moves import queue
 from threading import Thread
 
@@ -43,7 +44,8 @@ class ApexExecutor(RayExecutor):
     def __init__(self, environment_spec, agent_config, discard_queued_samples=False):
         """
         Args:
-            environment_spec (dict): Environment spec. Each worker in the cluster will instantiate
+            environment_spec (dict, callable): Environment spec or callable creating
+            an environment. Each worker in the cluster will instantiate
                 an environment using this spec.
             agent_config (dict): Config dict containing agent and execution specs.
             discard_queued_samples (bool): If true, discard samples if the learner queue is full instead
@@ -74,7 +76,7 @@ class ApexExecutor(RayExecutor):
 
         # Necessary for target network updates.
         self.weight_syncs_executed = 0
-        self.steps_since_weights_synced = dict()
+        self.steps_since_weights_synced = {}
 
         # These are the tasks actually interacting with the environment.
         self.env_sample_tasks = RayTaskPool()
@@ -86,22 +88,25 @@ class ApexExecutor(RayExecutor):
         self.setup_execution()
 
     def setup_execution(self):
-        # Start Ray cluster and connect to it.
-        self.ray_init()
-
         # Create local worker agent according to spec.
         # Extract states and actions space.
-        environment = RayExecutor.build_env_from_config(self.environment_spec)
+        environment = None
+        if isinstance(self.environment_spec, dict):
+            environment = Environment.from_spec(self.environment_spec)
+        elif hasattr(self.environment_spec, '__call__'):
+            environment = self.environment_spec()
         self.agent_config["state_space"] = environment.state_space
         self.agent_config["action_space"] = environment.action_space
 
-        self.local_agent = self.build_agent_from_config(self.agent_config)
+        # Start Ray cluster and connect to it.
+        self.local_agent = Agent.from_spec(self.agent_config)
 
         # Set up worker thread for performing updates.
         self.update_worker = UpdateWorker(
             agent=self.local_agent,
             in_queue_size=self.executor_spec["learn_queue_size"]
         )
+        self.ray_init()
 
         # Create remote sample workers based on ray cluster spec.
         self.num_replay_workers = self.executor_spec["num_replay_workers"]
@@ -133,21 +138,10 @@ class ApexExecutor(RayExecutor):
         self.ray_env_sample_workers = self.create_remote_workers(
             RayValueWorker, self.num_sample_workers, self.agent_config,
             # *args
-            self.worker_spec, self.environment_spec, self.worker_frameskip
+            self.worker_spec, self.environment_spec, self.worker_frame_skip
         )
         self.init_tasks()
 
-    def test_worker_init(self):
-        """
-        Tests every worker for successful constructor call (which may otherwise fail silently.
-        """
-        for ray_worker in self.ray_env_sample_workers:
-            self.logger.info("Testing worker for successful init: {}".format(self.worker_ids[ray_worker]))
-            task = ray_worker.get_constructor_success.remote()
-            result = ray.get(task)
-            assert result is True, "ERROR: constructor failed, attribute returned: {}" \
-                                   "instead of True".format(result)
-
     def init_tasks(self):
         # Start learner thread.
         self.update_worker.start()
@@ -281,7 +275,7 @@ class UpdateWorker(Thread):
         memory_actor, sample_batch = self.input_queue.get()
 
         if sample_batch is not None:
-            loss, loss_per_item = self.agent.update(batch=sample_batch)
+            losses = self.agent.update(batch=sample_batch)
             # Just pass back indices for updating.
-            self.output_queue.put((memory_actor, sample_batch["indices"], loss_per_item))
+            self.output_queue.put((memory_actor, sample_batch["indices"], losses[1]))
             self.update_done = True
