@@ -18,8 +18,6 @@ from __future__ import division
 from __future__ import print_function
 
 from copy import deepcopy
-
-from rlgraph.execution.ray.ray_util import worker_exploration
 from six.moves import xrange as range_
 import logging
 import numpy as np
@@ -28,6 +26,7 @@ import time
 from rlgraph import get_distributed_backend
 from rlgraph.agents import Agent
 from rlgraph.environments import Environment
+from rlgraph.execution.ray.ray_util import worker_exploration
 
 if get_distributed_backend() == "ray":
     import ray
@@ -62,8 +61,8 @@ class RayExecutor(object):
         self.sample_iteration_throughputs = None
         self.update_iteration_throughputs = None
         self.iteration_times = None
-        self.worker_frameskip = worker_spec.get("frame_skip", 1)
-        self.env_internal_frame_skip = environment_spec.get("frameskip", 1)
+        self.worker_frame_skip = worker_spec.get("worker_frame_skip", 1)
+        self.env_internal_frame_skip = worker_spec.get("env_internal_frame_skip", 1)
 
         # Map worker objects to host ids.
         self.worker_ids = {}
@@ -90,18 +89,16 @@ class RayExecutor(object):
         Creates Ray actors for remote execution.
 
         Args:
-            cls (RayValueWorker): Actor class, must be an instance of RayValueWorker.
-            num_actors (int): Num
+            cls (Union[RayValueWorker, RayPolicyWorker]): RayActor class.
+            num_actors (int): Num RayActor to create.
             agent_config (dict): Agent config.
             worker_spec (dict): Worker spec.
-            *args (any): Arguments for RayValueWorker class.
+            *args (any): Arguments for RayActor class.
 
         Returns:
             list: Remote Ray actors.
         """
         workers = []
-        init_tasks = []
-
         cls_as_remote = cls.as_remote(num_cpus=self.num_cpus_per_worker, num_gpus=self.num_gpus_per_worker).remote
 
         # Create remote objects and schedule init tasks.
@@ -113,18 +110,21 @@ class RayExecutor(object):
             worker = cls_as_remote(deepcopy(agent_config), worker_spec, *args)
             self.worker_ids[worker] = "worker_{}".format(i)
             workers.append(worker)
-            build_result = worker.init_agent.remote()
-            init_tasks.append(build_result)
-
-        ready, not_ready = ray.wait(init_tasks, num_returns=len(init_tasks))
-
-        for i, res in enumerate(ready):
-            result = ray.get(res)
-            if result:
-                self.logger.info("Successfully built agent num {}.".format(i))
+            self.logger.info("Successfully built agent num {}.".format(i))
 
         return workers
 
+    def test_worker_init(self):
+        """
+        Tests every worker for successful constructor call (which may otherwise fail silently.
+        """
+        for ray_worker in self.ray_env_sample_workers:
+            self.logger.info("Testing worker for successful init: {}".format(self.worker_ids[ray_worker]))
+            task = ray_worker.get_constructor_success.remote()
+            result = ray.get(task)
+            assert result is True, "ERROR: constructor failed, attribute returned: {}" \
+                                   "instead of True".format(result)
+
     def setup_execution(self):
         """
         Creates and initializes all remote agents on the Ray cluster. Does not
@@ -144,6 +144,7 @@ class RayExecutor(object):
         """
         Executes a workload on Ray and measures worker statistics. Workload semantics
         are decided via the private implementer, _execute_step().
+
         Args:
             workload (dict): Workload parameters, primarily 'num_timesteps' and 'report_interval'
                 to indicate how many steps to execute and how often to report results.
@@ -216,7 +217,7 @@ class RayExecutor(object):
             ops_per_second=(timesteps_executed / total_time),
             # Multiply sample throughput by these env_frames = samples * env_internal * worker_frame_skip:
             env_internal_frame_skip=self.env_internal_frame_skip,
-            worker_frame_skip=self.worker_frameskip,
+            worker_frame_skip=self.worker_frame_skip,
             min_iteration_sample_throughput=np.min(self.sample_iteration_throughputs),
             max_iteration_sample_throughput=np.max(self.sample_iteration_throughputs),
             mean_iteration_sample_throughput=np.mean(self.sample_iteration_throughputs),
@@ -268,25 +269,9 @@ class RayExecutor(object):
         agent_cls = Agent.__lookup_classes__.get(config.pop('type'))
         return agent_cls(**config)
 
-    @staticmethod
-    def build_env_from_config(env_spec):
-        """
-        Builds environment without using from_spec as Ray cannot handle kwargs correctly
-        at the moment.
-
-        Args:
-            env_spec (dict): Environment specification. Must contain 'type' field to lookup constructor.
-
-        Returns:
-            Environment: Env object.
-        """
-        env_spec = deepcopy(env_spec)
-        env_cls = Environment.lookup_class(env_spec.pop("type"))
-        return env_cls(**env_spec)
-
     def result_by_worker(self, worker_index=None):
         """
-        Retreives full episode-reward time series for a worker by id (or first worker in registry if None).
+        Retrieves full episode-reward time series for a worker by id (or first worker in registry if None).
 
         Args:
             worker_index (Optional[int]): Index of worker to fetch.
@@ -330,7 +315,7 @@ class RayExecutor(object):
 
     def get_sample_worker_ids(self):
         """
-        Returns identifeirs of all sample workers.
+        Returns identifiers of all sample workers.
 
         Returns:
             list: List of worker name strings in case individual analysis of one worker's results are required via
