@@ -19,6 +19,7 @@ from __future__ import print_function
 
 from copy import deepcopy
 import numpy as np
+from rlgraph.utils import util
 from six.moves import xrange as range_
 import time
 
@@ -36,10 +37,12 @@ if get_distributed_backend() == "ray":
 
 class RayPolicyWorker(RayActor):
     """
-    A ray policy worker for distributed policy optimisation.
+    A Ray policy worker for distributed policy optimisation. Unlike the value worker,
+    this worker does not need to explicitly store next-states, thus reducing communication overhead and
+    memory usage in the object store.
     """
 
-    def __init__(self, agent_config, worker_spec, env_spec, frameskip=1, auto_build=False):
+    def __init__(self, agent_config, worker_spec, env_spec, frameskip=1):
         """
         Creates agent and environment for Ray worker.
 
@@ -51,20 +54,15 @@ class RayPolicyWorker(RayActor):
         """
         assert get_distributed_backend() == "ray"
         # Internal frameskip of env.
-        self.env_frame_skip = env_spec.get("frameskip", 1)
+        self.env_frame_skip = worker_spec.get("env_internal_frame_skip", 1)
         # Worker computes weights for prioritized sampling.
         worker_spec = deepcopy(worker_spec)
-        self.worker_sample_size = worker_spec.pop("worker_sample_size")
-        self.worker_computes_weights = worker_spec.pop("worker_computes_weights", True)
-
-        # Use GAE.
-        self.generalized_advantage_estimation = worker_spec.pop("generalized_advantage_estimation", True)
-        self.gae_lambda = worker_spec.pop("gae_lambda", 1.0)
-        self.compress = worker_spec.pop("compress_states", False)
-
         self.num_environments = worker_spec.pop("num_worker_environments", 1)
+        self.worker_sample_size = worker_spec.pop("worker_sample_size") * self.num_environments
+        self.worker_executes_postprocessing = worker_spec.pop("worker_executes_postprocessing", True)
+
+        self.compress = worker_spec.pop("compress_states", False)
         self.env_ids = ["env_{}".format(i) for i in range_(self.num_environments)]
-        self.auto_build = auto_build
         num_background_envs = worker_spec.pop("num_background_envs", 1)
 
         self.vector_env = SequentialVectorEnv(self.num_environments, env_spec, num_background_envs)
@@ -73,24 +71,7 @@ class RayPolicyWorker(RayActor):
         agent_config['state_space'] = self.vector_env.state_space
         agent_config['action_space'] = self.vector_env.action_space
 
-        ray_exploration = worker_spec.pop("ray_exploration", None)
-        self.worker_executes_exploration = worker_spec.pop("worker_executes_exploration", False)
-        self.ray_exploration_set = False
-        if ray_exploration is not None:
-            # Update worker with worker specific constant exploration value.
-            # TODO too many levels?
-            assert agent_config["exploration_spec"]["epsilon_spec"]["decay_spec"]["type"] == "constant_decay", \
-                "ERROR: If using Ray's constant exploration, exploration type must be 'constant_decay'."
-            if self.worker_executes_exploration:
-                agent_config["exploration_spec"] = None
-                self.exploration_epsilon = ray_exploration
-            else:
-                agent_config["exploration_spec"]["epsilon_spec"]["decay_spec"]["constant_value"] = ray_exploration
-                self.ray_exploration_set = True
-
-        self.discount = agent_config.get("discount", 0.99)
         # Python based preprocessor as image resizing is broken in TF.
-
         self.preprocessors = {}
         preprocessing_spec = agent_config.get("preprocessing_spec", None)
         self.is_preprocessed = {}
@@ -120,7 +101,6 @@ class RayPolicyWorker(RayActor):
 
         # To continue running through multiple exec calls.
         self.last_states = self.vector_env.reset_all()
-        self.agent.reset()
 
         self.zero_batched_state = np.zeros((1,) + self.agent.preprocessed_state_space.shape)
         self.zero_unbatched_state = np.zeros(self.agent.preprocessed_state_space.shape)
@@ -147,14 +127,6 @@ class RayPolicyWorker(RayActor):
     def as_remote(cls, num_cpus=None, num_gpus=None):
         return ray.remote(num_cpus=num_cpus, num_gpus=num_gpus)(cls)
 
-    def init_agent(self):
-        """
-        Builds the agent. This is done as a separate task because meta graph
-        generation can take long.
-        """
-        self.agent.build()
-        return True
-
     def setup_preprocessor(self, preprocessing_spec, in_space):
         if preprocessing_spec is not None:
             # TODO move ingraph for python component assembly.
@@ -181,30 +153,12 @@ class RayPolicyWorker(RayActor):
         """
         Sets up agent, potentially modifying its configuration via worker specific settings.
         """
-        sample_exploration = worker_spec.pop("sample_exploration", False)
-        # Adjust exploration for this worker.
-        if sample_exploration:
-            assert self.ray_exploration_set is False, "ERROR: Cannot sample exploration if ray exploration is used."
-            exploration_min_value = worker_spec.pop("exploration_min_value", 0.0)
-            epsilon_spec = agent_config["exploration_spec"]["epsilon_spec"]
-
-            if epsilon_spec is not None and "decay_spec" in epsilon_spec:
-                decay_from = epsilon_spec["decay_spec"]["from"]
-                assert decay_from >= exploration_min_value, \
-                    "Min value for exploration sampling must be smaller than" \
-                    "decay_from {} in exploration_spec but is {}.".format(decay_from, exploration_min_value)
-
-                # Sample a new initial epsilon from the interval [exploration_min_value, decay_from).
-                sampled_from = np.random.uniform(low=exploration_min_value, high=decay_from)
-                epsilon_spec["decay_spec"]["from"] = sampled_from
-
         # Worker execution spec may differ from controller/learner.
         worker_exec_spec = worker_spec.get("execution_spec", None)
         if worker_exec_spec is not None:
             agent_config.update(execution_spec=worker_exec_spec)
 
         # Build lazily per default.
-        agent_config.update(auto_build=self.auto_build)
         return RayExecutor.build_agent_from_config(agent_config)
 
     def execute_and_get_timesteps(
@@ -269,6 +223,7 @@ class RayPolicyWorker(RayActor):
 
             actions = self.get_action(states=self.preprocessed_states_buffer,
                                       use_exploration=use_exploration, apply_preprocessing=False)
+
             next_states, step_rewards, terminals, infos = self.vector_env.step(actions=actions)
             # Worker frameskip not needed as done in env.
             # for _ in range_(self.worker_frameskip):
@@ -442,7 +397,7 @@ class RayPolicyWorker(RayActor):
         """
         Post-processes policy trajectories.
         """
-        if self.generalized_advantage_estimation:
+        if self.worker_executes_postprocessing:
             rewards = self.agent.post_process(
                 dict(
                     states=states,
@@ -453,7 +408,9 @@ class RayPolicyWorker(RayActor):
             )
 
         if self.compress:
-            states = [ray_compress(state) for state in states]
+            env_dtype = self.vector_env.state_space.dtype
+            states = [ray_compress(np.asarray(state, dtype=util.convert_dtype(dtype=env_dtype, to='np')))
+                      for state in states]
         return dict(
             states=states,
             actions=actions,
@@ -462,23 +419,6 @@ class RayPolicyWorker(RayActor):
         ), len(rewards)
 
     def get_action(self, states, use_exploration, apply_preprocessing):
-        if self.worker_executes_exploration:
-            # Only once for all actions otherwise we would have to call a session anyway.
-            if np.random.random() <= self.exploration_epsilon:
-                if self.num_environments == 1:
-                    # Sample returns without batch dim -> wrap.
-                    action = [self.agent.action_space.sample(size=self.num_environments)]
-                else:
-                    action = self.agent.action_space.sample(size=self.num_environments)
-            else:
-                if self.num_environments == 1:
-                    action = [self.agent.get_action(states=states, use_exploration=use_exploration,
-                                                    apply_preprocessing=apply_preprocessing)]
-                else:
-                    action = self.agent.get_action(states=states, use_exploration=use_exploration,
-                                                    apply_preprocessing=apply_preprocessing)
-            return action
-        else:
-            return self.agent.get_action(states=states, use_exploration=use_exploration,
-                                         apply_preprocessing=apply_preprocessing)
-
+        action = self.agent.get_action(states=states, use_exploration=use_exploration,
+                                       apply_preprocessing=apply_preprocessing)
+        return [action] if action.shape == () else action
