@@ -20,6 +20,7 @@ from __future__ import print_function
 import os
 import base64
 import numpy as np
+from six import string_types
 from rlgraph import get_distributed_backend
 from rlgraph.utils.rlgraph_errors import RLGraphError
 
@@ -61,8 +62,8 @@ class RayTaskPool(object):
     """
 
     def __init__(self):
-        self.ray_tasks = dict()
-        self.ray_objects = dict()
+        self.ray_tasks = {}
+        self.ray_objects = {}
 
     def add_task(self, worker, ray_object_ids):
         """
@@ -90,7 +91,7 @@ class RayTaskPool(object):
         pending_tasks = list(self.ray_tasks)
         if pending_tasks:
             # This ray function checks tasks and splits into ready and non-ready tasks.
-            ready, not_ready = ray.wait(pending_tasks, num_returns=len(pending_tasks), timeout=10)
+            ready, not_ready = ray.wait(pending_tasks, num_returns=len(pending_tasks), timeout=0.01)
             for obj_id in ready:
                 yield (self.ray_tasks.pop(obj_id), self.ray_objects.pop(obj_id))
 
@@ -156,13 +157,14 @@ def split_local_non_local_agents(ray_agents):
 def ray_compress(data):
     data = pyarrow.serialize(data).to_buffer().to_pybytes()
     data = lz4.frame.compress(data)
-    data = base64.b64encode(data)
-
+    # Unclear why ascii decoding.
+    data = base64.b64encode(data).decode("ascii")
+    # data = base64.b64encode(data)
     return data
 
 
 def ray_decompress(data):
-    if isinstance(data, bytes):
+    if isinstance(data, bytes) or isinstance(data, string_types):
         data = base64.b64decode(data)
         data = lz4.frame.decompress(data)
         data = pyarrow.deserialize(data)
