@@ -19,6 +19,7 @@ from __future__ import print_function
 
 from copy import deepcopy
 import numpy as np
+from rlgraph.utils import util
 from six.moves import xrange as range_
 import time
 
@@ -41,7 +42,7 @@ class RayValueWorker(RayActor):
     such as Ape-X.
     """
 
-    def __init__(self, agent_config, worker_spec, env_spec, frameskip=1, auto_build=False):
+    def __init__(self, agent_config, worker_spec, env_spec, frameskip=1):
         """
         Creates agent and environment for Ray worker.
 
@@ -53,15 +54,16 @@ class RayValueWorker(RayActor):
         """
         assert get_distributed_backend() == "ray"
         # Internal frameskip of env.
-        self.env_frame_skip = env_spec.get("frameskip", 1)
+        self.env_frame_skip = worker_spec.get("env_internal_frame_skip", 1)
         # Worker computes weights for prioritized sampling.
         worker_spec = deepcopy(worker_spec)
-        self.worker_sample_size = worker_spec.pop("worker_sample_size")
-        self.worker_computes_weights = worker_spec.pop("worker_computes_weights", True)
-        self.n_step_adjustment = worker_spec.pop("n_step_adjustment", 1)
         self.num_environments = worker_spec.pop("num_worker_environments", 1)
+
+        # Make sample size proportional to num envs.
+        self.worker_sample_size = worker_spec.pop("worker_sample_size") * self.num_environments
+        self.worker_executes_postprocessing = worker_spec.pop("worker_executes_postprocessing", True)
+        self.n_step_adjustment = worker_spec.pop("n_step_adjustment", 1)
         self.env_ids = ["env_{}".format(i) for i in range_(self.num_environments)]
-        self.auto_build = auto_build
         num_background_envs = worker_spec.pop("num_background_envs", 1)
 
         # TODO from spec once we decided on generic vectorization.
@@ -118,7 +120,6 @@ class RayValueWorker(RayActor):
 
         # To continue running through multiple exec calls.
         self.last_states = self.vector_env.reset_all()
-        self.agent.reset()
 
         self.zero_batched_state = np.zeros((1,) + self.agent.preprocessed_state_space.shape)
         self.zero_unbatched_state = np.zeros(self.agent.preprocessed_state_space.shape)
@@ -145,14 +146,6 @@ class RayValueWorker(RayActor):
     def as_remote(cls, num_cpus=None, num_gpus=None):
         return ray.remote(num_cpus=num_cpus, num_gpus=num_gpus)(cls)
 
-    def init_agent(self):
-        """
-        Builds the agent. This is done as a separate task because meta graph
-        generation can take long.
-        """
-        self.agent.build()
-        return True
-
     def setup_preprocessor(self, preprocessing_spec, in_space):
         if preprocessing_spec is not None:
             # TODO move ingraph for python component assembly.
@@ -202,7 +195,6 @@ class RayValueWorker(RayActor):
             agent_config.update(execution_spec=worker_exec_spec)
 
         # Build lazily per default.
-        agent_config.update(auto_build=self.auto_build)
         return RayExecutor.build_agent_from_config(agent_config)
 
     def execute_and_get_timesteps(
@@ -523,7 +515,7 @@ class RayValueWorker(RayActor):
         weights = np.ones_like(rewards)
 
         # Compute loss-per-item.
-        if self.worker_computes_weights:
+        if self.worker_executes_postprocessing:
             # Next states were just collected, we batch process them here.
             _, loss_per_item = self.agent.post_process(
                 dict(
@@ -536,16 +528,19 @@ class RayValueWorker(RayActor):
                 )
             )
             weights = np.abs(loss_per_item) + SMALL_NUMBER
+        env_dtype = self.vector_env.state_space.dtype
+        compressed_states = [ray_compress(np.asarray(state, dtype=util.convert_dtype(dtype=env_dtype, to='np')))
+                             for state in states]
 
-        compressed_states = [ray_compress(state) for state in states]
-        compressed_next_states = compressed_states[self.n_step_adjustment:] + [ray_compress(next_s) for next_s in
-                                                   next_states[-self.n_step_adjustment:]]
+        compressed_next_states = compressed_states[self.n_step_adjustment:] + \
+                                 [ray_compress(np.asarray(next_s,dtype=util.convert_dtype(dtype=env_dtype, to='np')))
+                                  for next_s in next_states[-self.n_step_adjustment:]]
         return dict(
-            states=np.array(compressed_states),
+            states=compressed_states,
             actions=np.array(actions),
             rewards=np.array(rewards),
             terminals=np.array(terminals),
-            next_states=np.array(compressed_next_states),
+            next_states=compressed_next_states,
             importance_weights=np.array(weights)
         ), len(rewards)
 
