@@ -117,7 +117,7 @@ class SingleThreadedWorker(Worker):
                          frameskip=None, reset=True):
         return self._execute(
             num_episodes=num_episodes,
-            max_timesteps_per_episode = max_timesteps_per_episode,
+            max_timesteps_per_episode=max_timesteps_per_episode,
             use_exploration=use_exploration,
             update_spec=update_spec,
             frameskip=frameskip,
@@ -235,7 +235,12 @@ class SingleThreadedWorker(Worker):
                     "ERROR: Cannot flip container-action batch with dict keys if returned value is not a dict OR " \
                     "values of returned value are not np.ndarrays!"
                 # TODO: What if actions come as nested dicts (more than one level deep)?
-                env_actions = [{key: value[i] for key, value in actions.items()} for i in range(len(actions[some_key]))]
+                if hasattr(actions[some_key], "len"):
+                    env_actions = [{key: value[i] for key, value in actions.items()} for i in range(len(actions[some_key]))]
+                else:
+                    # Action was not array type.
+                    env_actions = [{key: value for key, value in actions.items()}]
+
             # No flipping necessary.
             else:
                 env_actions = actions
@@ -243,7 +248,7 @@ class SingleThreadedWorker(Worker):
                     env_actions = [env_actions]
 
             for _ in range_(frameskip):
-                next_states, step_rewards, episode_terminals, infos = self.vector_env.step(actions=env_actions)
+                next_states, step_rewards, episode_terminals, _ = self.vector_env.step(actions=env_actions)
 
                 self.env_frames += self.num_environments
                 for i, step_reward in enumerate(step_rewards):
@@ -299,12 +304,9 @@ class SingleThreadedWorker(Worker):
                 if self.worker_executes_preprocessing and self.preprocessors[env_id] is not None:
                     #next_state = self.agent.state_space.force_batch(env_states[i])
                     next_states[i] = np.array(self.preprocessors[env_id].preprocess(env_states[i]))  # next_state
-                # TODO: If worker does not execute preprocessing, next state is not preprocessed here.
-                # Observe per environment.
-                self.agent.observe(
-                    preprocessed_states=preprocessed_states[i], actions=env_actions[i], internals=[],
-                    rewards=env_rewards[i], next_states=next_states[i],
-                    terminals=episode_terminals[i], env_id=self.env_ids[i]
+                self._observe(
+                    self.env_ids[i], preprocessed_states[i], env_actions[i], env_rewards[i], next_states[i],
+                    episode_terminals[i]
                 )
             self.update_if_necessary()
             timesteps_executed += self.num_environments
@@ -367,3 +369,13 @@ class SingleThreadedWorker(Worker):
         self.logger.info("Final episode reward: {}".format(results['final_episode_reward']))
 
         return results
+
+    def _observe(self, env_ids, states, actions, rewards, next_states, terminals):
+        # TODO: If worker does not execute preprocessing, next state is not preprocessed here.
+        # Observe per environment.
+        self.agent.observe(
+            preprocessed_states=states, actions=actions, internals=[],
+            rewards=rewards, next_states=next_states,
+            terminals=terminals, env_id=env_ids
+        )
+
