@@ -19,7 +19,9 @@ from __future__ import print_function
 
 import numpy as np
 
+from rlgraph import get_backend
 from rlgraph.spaces.box_space import BoxSpace
+from rlgraph.spaces.float_box import FloatBox
 from rlgraph.utils.util import convert_dtype as dtype_, LARGE_INTEGER
 
 
@@ -38,9 +40,11 @@ class IntBox(BoxSpace):
         NOTE: The `high` value for IntBoxes is excluded. Valid values thus are from the interval: [low,high[
         """
         if low is None:
-            assert high is None, "ERROR: If `low` is None, `high` must be None as well!"
-            low = -LARGE_INTEGER
-            high = LARGE_INTEGER
+            if high is not None:
+                low = 0
+            else:
+                low = -LARGE_INTEGER
+                high = LARGE_INTEGER
         # support calls like (IntBox(5) -> low=0, high=5)
         elif high is None:
             high = low
@@ -66,6 +70,19 @@ class IntBox(BoxSpace):
             return shape + ((self.num_categories,) if self.num_categories is not None else ())
         return shape
 
+    def as_one_hot_float_space(self):
+        """
+        Returns a new FloatBox Space resulting from one-hot flattening out this space
+        along its number of categories.
+
+        Returns:
+            FloatBox: The resulting FloatBox Space (with the same batch and time-rank settings).
+        """
+        return FloatBox(
+            low=0.0, high=1.0, shape=self.get_shape(with_category_rank=True),
+            add_batch_rank=self.has_batch_rank, add_time_rank=self.has_time_rank
+        )
+
     @property
     def flat_dim_with_categories(self):
         """
@@ -85,6 +102,18 @@ class IntBox(BoxSpace):
 
         return np.asarray(sample_, dtype=self.dtype)
 
+    def get_variable(
+            self, name, is_input_feed=False, add_batch_rank=None, add_time_rank=None, time_major=None, is_python=False,
+            local=False, **kwargs
+    ):
+        variable = super(IntBox, self).get_variable(
+            name, is_input_feed, add_batch_rank, add_time_rank, time_major, is_python, local, **kwargs
+        )
+        # Add num_categories information to placeholder.
+        if get_backend() == "tf" and is_python is False:
+            variable._num_categories = self.num_categories
+        return variable
+
     def contains(self, sample):
         # If int: Check for int type in given sample.
         if not np.equal(np.mod(sample, 1), 0).all():
