@@ -17,17 +17,17 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from rlgraph import get_backend
-from six.moves import xrange as range_
 import numpy as np
+from six.moves import xrange as range_
 
+from rlgraph import get_backend
 from rlgraph.spaces.bool_box import BoolBox
 from rlgraph.spaces.box_space import BoxSpace
 from rlgraph.spaces.containers import Dict, Tuple
 from rlgraph.spaces.float_box import FloatBox
 from rlgraph.spaces.int_box import IntBox
 from rlgraph.spaces.text_box import TextBox
-from rlgraph.utils.util import RLGraphError, convert_dtype, get_shape, LARGE_INTEGER
+from rlgraph.utils.util import RLGraphError, convert_dtype, get_shape, LARGE_INTEGER, force_tuple
 
 if get_backend() == "pytorch":
     import torch
@@ -167,6 +167,8 @@ def get_space_from_op(op):
                     shape = shape[1:]
                 add_batch_rank = True
 
+            # TODO: If op._batch_rank and/or op._time_rank are not set, set them now.
+
             base_dtype = op.dtype.base_dtype if hasattr(op.dtype, "base_dtype") else op.dtype
             # PyTorch does not have a bool type
             if get_backend() == "pytorch":
@@ -180,7 +182,8 @@ def get_space_from_op(op):
                                 time_major=time_major, dtype=convert_dtype(base_dtype, "np"))
             # IntBox
             elif "int" in base_dtype_str:
-                return IntBox(shape=shape, add_batch_rank=add_batch_rank, add_time_rank=add_time_rank,
+                high = getattr(op, "_num_categories", None)
+                return IntBox(high, shape=shape, add_batch_rank=add_batch_rank, add_time_rank=add_time_rank,
                               time_major=time_major, dtype=convert_dtype(base_dtype, "np"))
             # a BoolBox
             elif "bool" in base_dtype_str:
@@ -199,7 +202,7 @@ def sanity_check_space(
         must_have_batch_rank=None, must_have_time_rank=None, must_have_batch_or_time_rank=False,
         must_have_categories=None, num_categories=None,
         must_have_lower_limit=None, must_have_upper_limit=None,
-        rank=None
+        rank=None, shape=None
 ):
     """
     Sanity checks a given Space for certain criteria and raises exceptions if they are not met.
@@ -238,6 +241,8 @@ def sanity_check_space(
         rank (Optional[int,tuple]): An int or a tuple (min,max) range within which the Space's rank must lie.
             None if it doesn't matter.
 
+        shape (Optional[tuple[int]]): A tuple of ints specifying the required shape. None if it doesn't matter.
+
     Raises:
         RLGraphError: Various RLGraphErrors, if any of the conditions is not met.
     """
@@ -245,22 +250,22 @@ def sanity_check_space(
 
     # Check the types.
     if allowed_types is not None:
-        if not isinstance(space, tuple(allowed_types)):
+        if not isinstance(space, force_tuple(allowed_types)):
             raise RLGraphError("ERROR: Space ({}) is not an instance of {}!".format(space, allowed_types))
 
     if allowed_sub_types is not None:
         for flat_key, sub_space in flattened_space.items():
-            if not isinstance(sub_space, tuple(allowed_sub_types)):
+            if not isinstance(sub_space, force_tuple(allowed_sub_types)):
                 raise RLGraphError("ERROR: sub-Space '{}' ({}) is not an instance of "
                                    "{}!".format(flat_key, sub_space, allowed_sub_types))
 
     if non_allowed_types is not None:
-        if isinstance(space, tuple(non_allowed_types)):
+        if isinstance(space, force_tuple(non_allowed_types)):
             raise RLGraphError("ERROR: Space ({}) must not be an instance of {}!".format(space, non_allowed_types))
 
     if non_allowed_sub_types is not None:
         for flat_key, sub_space in flattened_space.items():
-            if isinstance(sub_space, tuple(non_allowed_sub_types)):
+            if isinstance(sub_space, force_tuple(non_allowed_sub_types)):
                 raise RLGraphError("ERROR: sub-Space '{}' ({}) must not be an instance of "
                                    "{}!".format(flat_key, sub_space, non_allowed_sub_types))
 
@@ -339,6 +344,14 @@ def sanity_check_space(
                         "ERROR: A Space (flat-key={}) of '{}' has rank {}, but its rank must be between {} and "
                         "{}!".format(flat_key, space, sub_space.rank, rank[0], rank[1]))
 
+    if shape is not None:
+        for flat_key, sub_space in flattened_space.items():
+            if sub_space.shape != shape:
+                raise RLGraphError(
+                    "ERROR: A Space (flat-key={}) of '{}' has shape {}, but its shape must be "
+                    "{}!".format(flat_key, space, sub_space.get_shape(), shape)
+                )
+
     if num_categories is not None:
         for flat_key, sub_space in flattened_space.items():
             if not isinstance(sub_space, IntBox):
@@ -425,8 +438,64 @@ def try_space_inference_from_list(list_op):
                 inner_shape = len(elem)
                 return BoxSpace.from_spec(spec=convert_dtype(float, "np"), shape=(batch_shape, inner_shape),
                                           add_batch_rank=True)
+            elif isinstance(elem, int):
+                return IntBox.from_spec(spec=int, shape=(batch_shape,), add_batch_rank=True)
+            elif isinstance(elem, float):
+                return FloatBox.from_spec(spec=int, shape=(batch_shape,), add_batch_rank=True)
         else:
             # Most general guess is a Float box.
             return FloatBox(shape=(batch_shape,))
     else:
-        raise ValueError("List inference should only be attempted on the Python backend.")
\ No newline at end of file
+        raise ValueError("List inference should only be attempted on the Python backend.")
+
+
+def get_default_distribution_from_space(box_space, bounded_distribution_type="beta",
+                                        discrete_distribution_type="categorical", gumbel_softmax_temperature=1.0):
+    """
+    Args:
+        box_space (BoxSpace): The primitive Space for which to derive a default distribution spec.
+        bounded_distribution_type (str): The lookup class string for a bounded FloatBox distribution.
+            Default: "beta".
+        discrete_distribution_type(str): The class of distributions to use for discrete action spaces. For options
+            check the components.distributions package. Default: categorical. Agents requiring reparameterization
+            may require a GumbelSoftmax distribution instead.
+        gumbel_softmax_temperature (float): Temperature parameter for the Gumbel-Softmax distribution used
+            for discrete actions.
+
+    Returns:
+        Dict: A Spec dict, from which a valid default distribution object can be created.
+    """
+    # IntBox: Categorical.
+    if isinstance(box_space, IntBox):
+        if discrete_distribution_type == "gumbel_softmax":
+            return dict(type="gumbel_softmax", temperature=gumbel_softmax_temperature)
+        else:
+            return dict(type=discrete_distribution_type)
+    # BoolBox: Bernoulli.
+    elif isinstance(box_space, BoolBox):
+        return dict(type="bernoulli")
+    # Continuous action space: Normal/Beta/etc. distribution.
+    elif isinstance(box_space, FloatBox):
+        # Unbounded -> Normal distribution.
+        if not is_bounded_space(box_space):
+            return dict(type="normal")
+        # Bounded -> according to the bounded_distribution parameter.
+        else:
+            return dict(type=bounded_distribution_type, low=box_space.low, high=box_space.high)
+
+
+def is_bounded_space(box_space):
+    if not isinstance(box_space, FloatBox):
+        return False
+    # Unbounded.
+    if box_space.low == float("-inf") and box_space.high == float("inf"):
+        return False
+    # Bounded.
+    elif box_space.low != float("-inf") and box_space.high != float("inf"):
+        return True
+    # TODO: Semi-bounded -> Exponential distribution.
+    else:
+        raise RLGraphError(
+            "Semi-bounded spaces for distribution-generation are not supported yet! You passed in low={} high={}.".
+            format(box_space.low, box_space.high)
+        )
