@@ -22,7 +22,7 @@ from six.moves import xrange as range_
 import unittest
 
 from rlgraph.components.component import Component
-from rlgraph.components.action_adapters.action_adapter import ActionAdapter
+from rlgraph.components.action_adapters.categorical_distribution_adapter import CategoricalDistributionAdapter
 from rlgraph.components.explorations.exploration import Exploration, EpsilonExploration
 from rlgraph.components.distributions import Categorical, Normal
 from rlgraph.spaces import *
@@ -41,7 +41,7 @@ class TestExplorations(unittest.TestCase):
 
         # Our distribution to go into the Exploration object.
         distribution = Categorical()
-        action_adapter = ActionAdapter(action_space=action_space)
+        action_adapter = CategoricalDistributionAdapter(action_space=action_space)
 
         exploration = Exploration.from_spec(dict(
             epsilon_spec=dict(
@@ -59,8 +59,8 @@ class TestExplorations(unittest.TestCase):
 
         @rlgraph_api(component=exploration_pipeline)
         def get_action(self_, nn_output, time_step):
-            out = action_adapter.get_logits_probabilities_log_probs(nn_output)
-            sample = distribution.sample_deterministic(out["probabilities"])
+            out = action_adapter.get_logits_parameters_log_probs(nn_output)
+            sample = distribution.sample_deterministic(out["parameters"])
             action = exploration.get_action(sample, time_step)
             return action
 
@@ -116,9 +116,9 @@ class TestExplorations(unittest.TestCase):
         distribution_a = Categorical(scope="d_a")
         distribution_b = Categorical(scope="d_b")
         distribution_c = Categorical(scope="d_c")
-        action_adapter_a = ActionAdapter(action_space=action_space["a"], scope="aa_a")
-        action_adapter_b = ActionAdapter(action_space=action_space["b"], scope="aa_b")
-        action_adapter_c = ActionAdapter(action_space=action_space["c"], scope="aa_c")
+        action_adapter_a = CategoricalDistributionAdapter(action_space=action_space["a"], scope="aa_a")
+        action_adapter_b = CategoricalDistributionAdapter(action_space=action_space["b"], scope="aa_b")
+        action_adapter_c = CategoricalDistributionAdapter(action_space=action_space["c"], scope="aa_c")
 
         exploration = Exploration.from_spec(dict(
             epsilon_spec=dict(
@@ -139,12 +139,12 @@ class TestExplorations(unittest.TestCase):
 
         @rlgraph_api(component=exploration_pipeline)
         def get_action(self_, nn_output, time_step):
-            out_a = action_adapter_a.get_logits_probabilities_log_probs(nn_output)
-            out_b = action_adapter_b.get_logits_probabilities_log_probs(nn_output)
-            out_c = action_adapter_c.get_logits_probabilities_log_probs(nn_output)
-            sample_a = distribution_a.sample_deterministic(out_a["probabilities"])
-            sample_b = distribution_b.sample_deterministic(out_b["probabilities"])
-            sample_c = distribution_c.sample_deterministic(out_c["probabilities"])
+            out_a = action_adapter_a.get_logits_parameters_log_probs(nn_output)
+            out_b = action_adapter_b.get_logits_parameters_log_probs(nn_output)
+            out_c = action_adapter_c.get_logits_parameters_log_probs(nn_output)
+            sample_a = distribution_a.sample_deterministic(out_a["parameters"])
+            sample_b = distribution_b.sample_deterministic(out_b["parameters"])
+            sample_c = distribution_c.sample_deterministic(out_c["parameters"])
             sample = self_._graph_fn_merge_actions(sample_a, sample_b, sample_c)
             action = exploration.get_action(sample, time_step)
             return action
@@ -159,11 +159,12 @@ class TestExplorations(unittest.TestCase):
             action_space=action_space
         )
 
-        # With exploration: Check, whether actions are equally distributed.
+        # With (random) exploration (in the beginning of decay interval): Check, whether actions are equally and
+        # more or less randomly distributed.
         batch_size = 2
         num_time_steps = 30
         nn_outputs = nn_output_space.sample(batch_size)
-        time_steps = time_step_space.sample(num_time_steps)
+        time_steps = np.maximum(time_step_space.sample(num_time_steps) - 8000, 0)
         # Collect action-batch-of-2 for each of our various random time steps.
         actions_a = np.ndarray(shape=(num_time_steps, batch_size), dtype=np.int)
         actions_b = np.ndarray(shape=(num_time_steps, batch_size), dtype=np.int)
@@ -238,7 +239,7 @@ class TestExplorations(unittest.TestCase):
 
         @rlgraph_api(component=exploration_pipeline)
         def get_action(self_, nn_output):
-            _, parameters, _ = action_adapter.get_logits_probabilities_log_probs(nn_output)
+            _, parameters, _ = action_adapter.get_logits_parameters_log_probs(nn_output)
             sample_stochastic = distribution.sample_stochastic(parameters)
             sample_deterministic = distribution.sample_deterministic(parameters)
             action = exploration.get_action(sample_stochastic, sample_deterministic)
