@@ -18,13 +18,14 @@ from __future__ import division
 from __future__ import print_function
 
 import unittest
+
 import numpy as np
 
 from rlgraph.components.layers.nn import NNLayer, DenseLayer, Conv2DLayer, ConcatLayer, MaxPool2DLayer, \
     LSTMLayer, ResidualLayer, LocalResponseNormalizationLayer
-from rlgraph.spaces import FloatBox
+from rlgraph.spaces import FloatBox, Dict
 from rlgraph.tests import ComponentTest
-from rlgraph.utils.numpy import sigmoid, relu, lstm_layer, dense_layer
+from rlgraph.utils.numpy import sigmoid, relu, lstm_layer
 
 
 class TestNNLayer(unittest.TestCase):
@@ -129,11 +130,11 @@ class TestNNLayer(unittest.TestCase):
         expected = np.array([[[[item0_ch0, item0_ch1, item0_ch2]]], [[[item1_ch0, item1_ch1, item1_ch2]]]])
         test.test(("apply", input_), expected_outputs=expected)
 
-
     def test_local_response_normalization_layer(self):
         space = FloatBox(shape=(2, 2, 3), add_batch_rank=True)  # e.g. a simple 3-color image
 
         # Todo: This is a very simple example ignoring the depth radius, which is the main idea of this normalization
+        # Also, 0.0 depth_radius doesn't run on GPU with cuDNN
         depth_radius = 0.0
         bias = np.random.random() + 1.0
         alpha = np.random.random() + 1.0
@@ -152,7 +153,6 @@ class TestNNLayer(unittest.TestCase):
         expected = np.array(calculated)
         test.test(("apply", input_), expected_outputs=expected)
 
-
     def test_concat_layer(self):
         # Spaces must contain batch dimension (otherwise, NNlayer will complain).
         space0 = FloatBox(shape=(2, 3), add_batch_rank=True)
@@ -164,16 +164,29 @@ class TestNNLayer(unittest.TestCase):
 
         # Batch of 2 samples to concatenate.
         inputs = (
-            np.array([[[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], [[1.1, 2.1, 3.1], [4.1, 5.1, 6.1]]]),
-            np.array([[[1.0], [2.0]], [[3.0], [4.0]]]),
-            np.array([[[1.2, 2.2], [3.2, 4.2]], [[1.3, 2.3], [3.3, 4.3]]])
+            np.array([[[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], [[1.1, 2.1, 3.1], [4.1, 5.1, 6.1]]], dtype=np.float32),
+            np.array([[[1.0], [2.0]], [[3.0], [4.0]]], dtype=np.float32),
+            np.array([[[1.2, 2.2], [3.2, 4.2]], [[1.3, 2.3], [3.3, 4.3]]], dtype=np.float32)
         )
-        expected = np.array([[[1.0, 2.0, 3.0, 1.0, 1.2, 2.2],
-                              [4.0, 5.0, 6.0, 2.0, 3.2, 4.2]],
-                             [[1.1, 2.1, 3.1, 3.0, 1.3, 2.3],
-                              [4.1, 5.1, 6.1, 4.0, 3.3, 4.3]]], dtype=np.float32)
+        expected = np.concatenate((inputs[0], inputs[1], inputs[2]), axis=-1)
         test.test(("apply", inputs), expected_outputs=expected)
 
+    def test_concat_layer_with_dict_input(self):
+        # Spaces must contain batch dimension (otherwise, NNlayer will complain).
+        input_space = Dict({
+            "a": FloatBox(shape=(2, 3)),
+            "b": FloatBox(shape=(2, 1)),
+            "c": FloatBox(shape=(2, 2)),
+        }, add_batch_rank=True)
+
+        concat_layer = ConcatLayer(dict_keys=["c", "a", "b"])  # some crazy order
+        test = ComponentTest(component=concat_layer, input_spaces=dict(inputs=input_space))
+
+        # Batch of n samples to concatenate.
+        inputs = input_space.sample(4)
+        expected = np.concatenate((inputs["c"], inputs["a"], inputs["b"]), axis=-1)
+        test.test(("apply", tuple([inputs])), expected_outputs=expected)
+
     def test_residual_layer(self):
         # Input space to residual layer (with 2-repeat [simple Conv2D layer]-residual-unit).
         input_space = FloatBox(shape=(2, 2, 3), add_batch_rank=True)
@@ -215,7 +228,7 @@ class TestNNLayer(unittest.TestCase):
         inputs = np.ones(shape=(batch_size, sequence_length, 3))
 
         # First matmul the inputs times the LSTM matrix:
-        var_values = test.read_variable_values(lstm_layer_component.variables)
+        var_values = test.read_variable_values(lstm_layer_component.variable_registry)
         lstm_matrix = var_values["lstm-layer/lstm-cell/kernel"]
         lstm_biases = var_values["lstm-layer/lstm-cell/bias"]
 
