@@ -36,7 +36,6 @@ class TestPolicies(unittest.TestCase):
 
         # action_space (5 possible actions).
         action_space = IntBox(5, add_batch_rank=True)
-        flat_float_action_space = FloatBox(shape=(5,), add_batch_rank=True)
 
         policy = Policy(network_spec=config_from_path("configs/test_simple_nn.json"), action_space=action_space)
         test = ComponentTest(
@@ -44,42 +43,40 @@ class TestPolicies(unittest.TestCase):
             input_spaces=dict(
                 nn_input=state_space,
                 actions=action_space,
-                logits=flat_float_action_space,
-                probabilities=flat_float_action_space,
-                parameters=flat_float_action_space
             ),
             action_space=action_space
         )
-        policy_params = test.read_variable_values(policy.variables)
+        policy_params = test.read_variable_values(policy.variable_registry)
 
         # Some NN inputs (4 input nodes, batch size=2).
         states = np.array([[-0.08, 0.4, -0.05, -0.55], [13.0, -14.0, 10.0, -16.0]])
         # Raw NN-output.
-        expected_nn_output = np.matmul(states, policy_params["policy/test-network/hidden-layer/dense/kernel"])
-        test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=6)
+        expected_nn_output = np.matmul(states,
+                                       ComponentTest.read_params("policy/test-network/hidden-layer", policy_params))
+
+        test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=5)
 
         # Raw action layer output; Expected shape=(2,5): 2=batch, 5=action categories
         expected_action_layer_output = np.matmul(
-            expected_nn_output, policy_params["policy/action-adapter-0/action-network/action-layer/dense/kernel"]
+            expected_nn_output,
+            ComponentTest.read_params("policy/action-adapter-0/action-network/action-layer", policy_params)
         )
         test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_output),
                   decimals=5)
 
         # Logits, parameters (probs) and skip log-probs (numerically unstable for small probs).
-        expected_probabilities_output = softmax(expected_action_layer_output, axis=-1)
-        test.test(("get_logits_probabilities_log_probs", states, ["logits", "probabilities"]), expected_outputs=dict(
-            logits=expected_action_layer_output, probabilities=np.array(expected_probabilities_output, dtype=np.float32)
+        expected_parameters_output = softmax(expected_action_layer_output, axis=-1)
+        test.test(("get_logits_parameters_log_probs", states, ["logits", "parameters"]), expected_outputs=dict(
+            logits=expected_action_layer_output, parameters=np.array(expected_parameters_output, dtype=np.float32)
         ), decimals=5)
 
-        print("Probs: {}".format(expected_probabilities_output))
-
         expected_actions = np.argmax(expected_action_layer_output, axis=-1)
-        test.test(("get_action", states), expected_outputs=dict(action=expected_actions))
+        test.test(("get_action", states, ["action"]), expected_outputs=dict(action=expected_actions))
 
         # Action log-probs.
         expected_action_log_prob_output = np.log(np.array([
-            expected_probabilities_output[0][expected_actions[0]],
-            expected_probabilities_output[1][expected_actions[1]],
+            expected_parameters_output[0][expected_actions[0]],
+            expected_parameters_output[1][expected_actions[1]],
         ]))
         test.test(("get_action_log_probs", [states, expected_actions]),
                   expected_outputs=dict(action_log_probs=expected_action_log_prob_output,
@@ -87,12 +84,12 @@ class TestPolicies(unittest.TestCase):
 
         # Stochastic sample.
         out = test.test(("get_stochastic_action", states), expected_outputs=None)  # dict(action=expected_actions))
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (2,))
 
         # Deterministic sample.
         test.test(("get_deterministic_action", states), expected_outputs=None)  # dict(action=expected_actions))
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (2,))
 
         # Distribution's entropy.
@@ -102,8 +99,8 @@ class TestPolicies(unittest.TestCase):
 
         # Action log-probs.
         expected_action_log_prob_output = dict(action_log_probs=np.log(np.array([
-            expected_probabilities_output[0][expected_actions[0]],
-            expected_probabilities_output[1][expected_actions[1]]
+            expected_parameters_output[0][expected_actions[0]],
+            expected_parameters_output[1][expected_actions[1]]
         ])), logits=expected_action_layer_output)
         test.test(("get_action_log_probs", [states, expected_actions]),
                   expected_outputs=expected_action_log_prob_output, decimals=5)
@@ -126,26 +123,27 @@ class TestPolicies(unittest.TestCase):
             input_spaces=dict(
                 nn_input=state_space,
                 actions=action_space,
-                probabilities=flat_float_action_space,
-                parameters=flat_float_action_space,
-                logits=flat_float_action_space
+                #parameters=flat_float_action_space,
+                #logits=flat_float_action_space
             ),
             action_space=action_space,
         )
-        policy_params = test.read_variable_values(shared_value_function_policy.variables)
+        policy_params = test.read_variable_values(shared_value_function_policy.variable_registry)
 
         # Some NN inputs (4 input nodes, batch size=3).
         states = state_space.sample(size=3)
         # Raw NN-output (3 hidden nodes). All weights=1.5, no biases.
         expected_nn_output = relu(np.matmul(
-            states, policy_params["shared-value-function-policy/test-network/hidden-layer/dense/kernel"]
+            states, ComponentTest.read_params("shared-value-function-policy/test-network/hidden-layer", policy_params)
         ), 0.1)
+
         test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=5)
 
         # Raw action layer output; Expected shape=(3,3): 3=batch, 2=action categories + 1 state value
         expected_action_layer_output = np.matmul(
             expected_nn_output,
-            policy_params["shared-value-function-policy/action-adapter-0/action-network/action-layer/dense/kernel"]
+            ComponentTest.read_params("shared-value-function-policy/action-adapter-0/action-network/action-layer/",
+                                      policy_params)
         )
         test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_output),
                   decimals=5)
@@ -153,36 +151,36 @@ class TestPolicies(unittest.TestCase):
         # State-values: One for each item in the batch.
         expected_state_value_output = np.matmul(
             expected_nn_output,
-            policy_params["shared-value-function-policy/value-function-node/dense-layer/dense/kernel"]
+            ComponentTest.read_params("shared-value-function-policy/value-function-node/dense-layer", policy_params)
         )
         test.test(("get_state_values", states), expected_outputs=dict(state_values=expected_state_value_output),
                   decimals=5)
 
         # Logits-values.
-        test.test(("get_state_values_logits_probabilities_log_probs", states, ["state_values", "logits"]),
+        test.test(("get_state_values_logits_parameters_log_probs", states, ["state_values", "logits"]),
                   expected_outputs=dict(state_values=expected_state_value_output, logits=expected_action_layer_output),
                   decimals=5)
 
         # Parameter (probabilities). Softmaxed logits.
-        expected_probabilities_output = softmax(expected_action_layer_output, axis=-1)
-        test.test(("get_logits_probabilities_log_probs", states, ["logits", "probabilities"]), expected_outputs=dict(
+        expected_parameters_output = softmax(expected_action_layer_output, axis=-1)
+        test.test(("get_logits_parameters_log_probs", states, ["logits", "parameters"]), expected_outputs=dict(
             logits=expected_action_layer_output,
-            probabilities=expected_probabilities_output
+            parameters=expected_parameters_output
         ), decimals=5)
 
-        print("Probs: {}".format(expected_probabilities_output))
+        print("Probs: {}".format(expected_parameters_output))
 
         expected_actions = np.argmax(expected_action_layer_output, axis=-1)
-        test.test(("get_action", states), expected_outputs=dict(action=expected_actions))
+        test.test(("get_action", states, ["action"]), expected_outputs=dict(action=expected_actions))
 
         # Stochastic sample.
         out = test.test(("get_stochastic_action", states), expected_outputs=None)
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (3,))
 
         # Deterministic sample.
         out = test.test(("get_deterministic_action", states), expected_outputs=None)
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (3,))
 
         # Distribution's entropy.
@@ -213,28 +211,29 @@ class TestPolicies(unittest.TestCase):
             input_spaces=dict(
                 nn_input=state_space,
                 actions=action_space,
-                probabilities=flat_float_action_space,
-                parameters=flat_float_action_space,
-                logits=flat_float_action_space
+                #parameters=flat_float_action_space,
+                #logits=flat_float_action_space
             ),
             action_space=action_space,
         )
-        policy_params = test.read_variable_values(shared_value_function_policy.variables)
+        policy_params = test.read_variable_values(shared_value_function_policy.variable_registry)
 
         # Some NN inputs.
         states = state_space.sample(size=(2, 3))
         states_folded = np.reshape(states, newshape=(6, 3))
         # Raw NN-output (3 hidden nodes). All weights=1.5, no biases.
         expected_nn_output = relu(np.matmul(
-            states_folded, policy_params["shared-value-function-policy/test-network/hidden-layer/dense/kernel"]
+            states_folded,
+            ComponentTest.read_params("shared-value-function-policy/test-network/hidden-layer", policy_params)
         ), 0.1)
         test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=5)
 
         # Raw action layer output; Expected shape=(3,3): 3=batch, 2=action categories + 1 state value
         expected_action_layer_output = np.matmul(
-            expected_nn_output, policy_params["shared-value-function-policy/action-adapter-0/action-network/"
-                                              "action-layer/dense/kernel"]
-        )
+            expected_nn_output,
+            ComponentTest.read_params("shared-value-function-policy/action-adapter-0/action-network/action-layer/",
+                                      policy_params))
+
         expected_action_layer_output = np.reshape(expected_action_layer_output, newshape=(2, 3, 4))
         test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_output),
                   decimals=5)
@@ -242,7 +241,7 @@ class TestPolicies(unittest.TestCase):
         # State-values: One for each item in the batch.
         expected_state_value_output = np.matmul(
             expected_nn_output,
-            policy_params["shared-value-function-policy/value-function-node/dense-layer/dense/kernel"]
+            ComponentTest.read_params("shared-value-function-policy/value-function-node/dense-layer", policy_params)
         )
         expected_state_value_output_unfolded = np.reshape(expected_state_value_output, newshape=(2, 3, 1))
         test.test(("get_state_values", states), expected_outputs=dict(state_values=expected_state_value_output_unfolded),
@@ -250,33 +249,33 @@ class TestPolicies(unittest.TestCase):
 
         expected_action_layer_output_unfolded = np.reshape(expected_action_layer_output, newshape=(2, 3, 4))
         test.test(
-            ("get_state_values_logits_probabilities_log_probs", states, ["state_values", "logits"]),
+            ("get_state_values_logits_parameters_log_probs", states, ["state_values", "logits"]),
             expected_outputs=dict(
                 state_values=expected_state_value_output_unfolded, logits=expected_action_layer_output_unfolded
             ), decimals=5
         )
 
         # Parameter (probabilities). Softmaxed logits.
-        expected_probabilities_output = softmax(expected_action_layer_output_unfolded, axis=-1)
-        test.test(("get_logits_probabilities_log_probs", states, ["logits", "probabilities"]), expected_outputs=dict(
+        expected_parameters_output = softmax(expected_action_layer_output_unfolded, axis=-1)
+        test.test(("get_logits_parameters_log_probs", states, ["logits", "parameters"]), expected_outputs=dict(
             logits=expected_action_layer_output_unfolded,
-            probabilities=expected_probabilities_output
+            parameters=expected_parameters_output
         ), decimals=5)
 
-        print("Probs: {}".format(expected_probabilities_output))
+        print("Probs: {}".format(expected_parameters_output))
 
         expected_actions = np.argmax(expected_action_layer_output_unfolded, axis=-1)
-        test.test(("get_action", states), expected_outputs=dict(action=expected_actions))
+        test.test(("get_action", states, ["action"]), expected_outputs=dict(action=expected_actions))
 
         # Action log-probs.
         expected_action_log_prob_output = np.log(np.array([[
-            expected_probabilities_output[0][0][expected_actions[0][0]],
-            expected_probabilities_output[0][1][expected_actions[0][1]],
-            expected_probabilities_output[0][2][expected_actions[0][2]],
+            expected_parameters_output[0][0][expected_actions[0][0]],
+            expected_parameters_output[0][1][expected_actions[0][1]],
+            expected_parameters_output[0][2][expected_actions[0][2]],
         ], [
-            expected_probabilities_output[1][0][expected_actions[1][0]],
-            expected_probabilities_output[1][1][expected_actions[1][1]],
-            expected_probabilities_output[1][2][expected_actions[1][2]],
+            expected_parameters_output[1][0][expected_actions[1][0]],
+            expected_parameters_output[1][1][expected_actions[1][1]],
+            expected_parameters_output[1][2][expected_actions[1][2]],
         ]]))
         test.test(("get_action_log_probs", [states, expected_actions]), expected_outputs=dict(
             action_log_probs=expected_action_log_prob_output, logits=expected_action_layer_output_unfolded
@@ -284,12 +283,12 @@ class TestPolicies(unittest.TestCase):
 
         # Deterministic sample.
         out = test.test(("get_deterministic_action", states), expected_outputs=None)
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (2, 3))  # Make sure output is unfolded.
 
         # Stochastic sample.
         out = test.test(("get_stochastic_action", states), expected_outputs=None)
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (2, 3))  # Make sure output is unfolded.
 
         # Distribution's entropy.
@@ -304,7 +303,7 @@ class TestPolicies(unittest.TestCase):
 
         # action_space (2 possible actions).
         action_space = IntBox(2, add_batch_rank=True)
-        flat_float_action_space = FloatBox(shape=(2,), add_batch_rank=True)
+        # flat_float_action_space = FloatBox(shape=(2,), add_batch_rank=True)
 
         # Policy with dueling logic.
         policy = DuelingPolicy(
@@ -322,72 +321,75 @@ class TestPolicies(unittest.TestCase):
             input_spaces=dict(
                 nn_input=nn_input_space,
                 actions=action_space,
-                probabilities=flat_float_action_space,
-                parameters=flat_float_action_space,
-                logits=flat_float_action_space
+                #parameters=flat_float_action_space,
+                #logits=flat_float_action_space
             ),
             action_space=action_space
         )
-        policy_params = test.read_variable_values(policy.variables)
+        policy_params = test.read_variable_values(policy.variable_registry)
 
         # Some NN inputs.
         nn_input = nn_input_space.sample(size=3)
         # Raw NN-output.
         expected_nn_output = relu(np.matmul(
-            nn_input, policy_params["dueling-policy/test-network/hidden-layer/dense/kernel"]), 0.1
+            nn_input,
+            ComponentTest.read_params("dueling-policy/test-network/hidden-layer", policy_params)), 0.1
         )
         test.test(("get_nn_output", nn_input), expected_outputs=dict(output=expected_nn_output))
 
         # Raw action layer output.
         expected_raw_advantages = np.matmul(relu(np.matmul(
-            expected_nn_output, policy_params["dueling-policy/action-adapter-0/action-network/dense-layer/dense/kernel"]
-        ), 0.1), policy_params["dueling-policy/action-adapter-0/action-network/action-layer/dense/kernel"])
+            expected_nn_output,
+            ComponentTest.read_params("dueling-policy/action-adapter-0/action-network/dense-layer", policy_params)
+        ), 0.1),
+            ComponentTest.read_params("dueling-policy/action-adapter-0/action-network/action-layer", policy_params))
         test.test(("get_action_layer_output", nn_input), expected_outputs=dict(output=expected_raw_advantages),
                   decimals=5)
 
         # Single state values.
         expected_state_values = np.matmul(relu(np.matmul(
             expected_nn_output,
-            policy_params["dueling-policy/dense-layer-state-value-stream/dense/kernel"]
-        )), policy_params["dueling-policy/state-value-node/dense/kernel"])
+            ComponentTest.read_params("dueling-policy/dense-layer-state-value-stream", policy_params)
+        )),
+            ComponentTest.read_params("dueling-policy/state-value-node", policy_params))
         test.test(("get_state_values", nn_input), expected_outputs=dict(state_values=expected_state_values),
                   decimals=5)
 
         # State-values: One for each item in the batch.
         expected_q_values_output = expected_state_values + expected_raw_advantages - \
             np.mean(expected_raw_advantages, axis=-1, keepdims=True)
-        test.test(("get_logits_probabilities_log_probs", nn_input, "logits"), expected_outputs=dict(
+        test.test(("get_logits_parameters_log_probs", nn_input, "logits"), expected_outputs=dict(
             logits=expected_q_values_output
         ), decimals=5)
 
         # Parameter (probabilities). Softmaxed q_values.
-        expected_probabilities_output = softmax(expected_q_values_output, axis=-1)
-        test.test(("get_logits_probabilities_log_probs", nn_input, ["logits", "probabilities"]), expected_outputs=dict(
-            logits=expected_q_values_output, probabilities=expected_probabilities_output
+        expected_parameters_output = softmax(expected_q_values_output, axis=-1)
+        test.test(("get_logits_parameters_log_probs", nn_input, ["logits", "parameters"]), expected_outputs=dict(
+            logits=expected_q_values_output, parameters=expected_parameters_output
         ), decimals=5)
 
-        print("Probs: {}".format(expected_probabilities_output))
+        print("Probs: {}".format(expected_parameters_output))
 
         expected_actions = np.argmax(expected_q_values_output, axis=-1)
-        test.test(("get_action", nn_input), expected_outputs=dict(action=expected_actions))
+        test.test(("get_action", nn_input, ["action"]), expected_outputs=dict(action=expected_actions))
 
         # Action log-probs.
         expected_action_log_prob_output = np.log(np.array([
-            expected_probabilities_output[0][expected_actions[0]],
-            expected_probabilities_output[1][expected_actions[1]],
-            expected_probabilities_output[2][expected_actions[2]],
+            expected_parameters_output[0][expected_actions[0]],
+            expected_parameters_output[1][expected_actions[1]],
+            expected_parameters_output[2][expected_actions[2]],
         ]))
         test.test(("get_action_log_probs", [nn_input, expected_actions]),
                   expected_outputs=dict(action_log_probs=expected_action_log_prob_output, logits=expected_q_values_output), decimals=5)
 
         # Stochastic sample.
         out = test.test(("get_stochastic_action", nn_input), expected_outputs=None)
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (3,))
 
         # Deterministic sample.
         out = test.test(("get_deterministic_action", nn_input), expected_outputs=None)
-        self.assertTrue(out["action"].dtype == np.int32)
+        self.assertTrue(out["action"].dtype == np.int32 or (out["action"].dtype == np.int64))
         self.assertTrue(out["action"].shape == (3,))
 
         # Distribution's entropy.
@@ -402,7 +404,7 @@ class TestPolicies(unittest.TestCase):
         nn_input_space = FloatBox(shape=(4,), add_batch_rank=True)
         action_space = FloatBox(low=-1.0, high=1.0, shape=(1,), add_batch_rank=True)
         # Double the shape for alpha/beta params.
-        action_space_parameters = FloatBox(shape=(2,), add_batch_rank=True)
+        # action_space_parameters = Tuple(FloatBox(shape=(1,)), FloatBox(shape=(1,)), add_batch_rank=True)
 
         policy = Policy(network_spec=config_from_path("configs/test_simple_nn.json"), action_space=action_space)
         test = ComponentTest(
@@ -410,35 +412,38 @@ class TestPolicies(unittest.TestCase):
             input_spaces=dict(
                 nn_input=nn_input_space,
                 actions=action_space,
-                logits=FloatBox(shape=(1,), add_batch_rank=True),
-                probabilities=FloatBox(add_batch_rank=True),
-                parameters=action_space_parameters,
+                #parameters=action_space_parameters,
+                #logits=FloatBox(shape=(1,), add_batch_rank=True)
             ),
             action_space=action_space
         )
 
-        policy_params = test.read_variable_values(policy.variables)
+        policy_params = test.read_variable_values(policy.variable_registry)
 
         # Some NN inputs.
         nn_input = nn_input_space.sample(size=3)
         # Raw NN-output.
-        expected_nn_output = np.matmul(nn_input, policy_params["policy/test-network/hidden-layer/dense/kernel"])
+        expected_nn_output = np.matmul(nn_input,
+                                       ComponentTest.read_params("policy/test-network/hidden-layer", policy_params))
         test.test(("get_nn_output", nn_input), expected_outputs=dict(output=expected_nn_output))
 
         # Raw action layer output.
         expected_raw_logits = np.matmul(
-            expected_nn_output, policy_params["policy/action-adapter-0/action-network/action-layer/dense/kernel"]
+            expected_nn_output,
+            ComponentTest.read_params("policy/action-adapter-0/action-network/action-layer", policy_params)
         )
         test.test(("get_action_layer_output", nn_input), expected_outputs=dict(output=expected_raw_logits),
                   decimals=5)
 
         # Parameter (alpha/betas).
-        expected_parameters_output = np.log(np.exp(expected_raw_logits) + 1.0) + 1.0
+        expected_alpha_parameters = np.log(np.exp(expected_raw_logits[:, 0:1]) + 1.0) + 1.0
+        expected_beta_parameters = np.log(np.exp(expected_raw_logits[:, 1:]) + 1.0) + 1.0
+        expected_parameters = tuple([expected_alpha_parameters, expected_beta_parameters])
         test.test(("get_logits_parameters_log_probs", nn_input, ["logits", "parameters"]), expected_outputs=dict(
-            logits=expected_raw_logits, parameters=expected_parameters_output
+            logits=expected_raw_logits, parameters=expected_parameters
         ), decimals=5)
 
-        print("Params: {}".format(expected_parameters_output))
+        print("Params: {}".format(expected_parameters))
 
         actions = test.test(("get_action", nn_input))["action"]
         self.assertTrue(actions.dtype == np.float32)
@@ -448,8 +453,11 @@ class TestPolicies(unittest.TestCase):
 
         # Action log-probs.
         actions_scaled_back = (actions + 1.0) / 2.0
-        expected_action_log_prob_output = np.log(beta.pdf(actions_scaled_back, expected_parameters_output[:, 1], expected_parameters_output[:, 0]))
-        expected_action_log_prob_output = np.array([[expected_action_log_prob_output[0][0]], [expected_action_log_prob_output[1][1]], [expected_action_log_prob_output[2][2]]])
+        expected_action_log_prob_output = np.log(
+            beta.pdf(actions_scaled_back, expected_alpha_parameters, expected_beta_parameters)
+        )
+        # expected_action_log_prob_output = np.array([[expected_action_log_prob_output[0][0]],
+        # [expected_action_log_prob_output[1][1]], [expected_action_log_prob_output[2][2]]])
         test.test(("get_action_log_probs", [nn_input, actions]),
                   expected_outputs=dict(action_log_probs=expected_action_log_prob_output,
                                         logits=expected_raw_logits), decimals=5)
