@@ -24,6 +24,7 @@ import inspect
 import re
 import sys
 from rlgraph import get_backend
+from rlgraph.utils.execution_util import define_by_run_flatten
 from rlgraph.utils.rlgraph_errors import RLGraphError
 
 if get_backend() == "tf":
@@ -31,8 +32,13 @@ if get_backend() == "tf":
 elif get_backend() == "pytorch":
     import torch
 
+# Some small floating point number. Can be used as a small epsilon for numerical stability purposes.
 SMALL_NUMBER = 1e-6
+# Some large int number. May be increased here, if needed.
 LARGE_INTEGER = 100000000
+# Min and Max outputs (clipped) from an NN-output layer interpreted as the log(stddev) of some loc + scale distribution.
+MIN_LOG_STDDEV = -20
+MAX_LOG_STDDEV = 2
 
 # Logging config for testing.
 logging_formatter = logging.Formatter('%(asctime)s:%(levelname)s:%(message)s', datefmt='%y-%m-%d %H:%M:%S')
@@ -376,8 +382,9 @@ def force_torch_tensors(params, requires_grad=False):
     if get_backend() == "pytorch":
         tensor_params = []
         for param in params:
-            # Only flat dicts for now.
             if isinstance(param, dict):
+                # Flatten dict.
+                param = define_by_run_flatten(param)
                 ret = {}
                 for key, value in param.items():
                     ret[key] = convert_param(value, requires_grad)
